--------------------------------------------------
-- LagrangeBox.lua
-- Edit by Lygin Gennady a.k.a Croc 30.01.2017
-- Edit by Patrick Ziegler a.k.a. Gironde 07-04-2017
--------------------------------------------------

--	Title: Lagrange Points
--
--	DESCRIPTION:
--		LagrangeBox.lua
--
--		This is the graphical (mouse driven) version of the Lagrange Point Tools.
--
--		A keystroke driven version of the Lagrange Point Tools is also available, suitable for use with any
--		standard installation of Celestia from version 1.6.1 onwards.
--
--	LICENCE:
-- 		Free for personal use only.
--		Any commercial, monetized, or public use or display of the Lagrange Points Tools, or any derivative 
--		works or versions is forbidden without explicit permission from the author.
--		
--	AUTHOR: 
--		Phil Batchelor
--		Copyright 2007-2012, All rights reserved
--			Contact me by posting a message at the Celestia Forum: http://shatters.net/forum
--			in this thread: 
--					http://shatters.net/forum/viewtopic.php?f=23&t=17130
--
--	SOFTWARE / HARDWARE REQUIREMENTS:
--		Requires Celestia version 1.6.1 or later.
--			See: http://shatters.net/celestia
--	
--		Requires prior installation of Lua Edu Tools v1.2beta8 
--			See: http://shatters.net/forum/viewtopic.php?f=11&t=13162
--
--		The Lagrange Points Tool was designed on a Windows PC, but should work on any platform on which Celestia runs.
--
--	INSTALLATION:
--		Instructions in: cc_lagrange_README.txt
--
--========================

require "locale"
require "closeButton"
require "lp_config"
-- #require "lp_locale"
----------------------------------------------
-- Set initial values
---------------------------------------------- 
local newwidth, newheight;
local gn=5
local gg=gn-1
local LagrangeCmdWidth, LagrangeCmdHeight = 186, 17*gn+2+2;
local obs = celestia:getobserver();
---------------------------------------------

	lp_boxWidth = lp_boxWidth or 186

	-- add the plugin translation to the global translations
	if pcall( function() require ("lp_locale_"..lang) end ) then
		for engl, transl in pairs(lp_loc_str) do
			loc_str[engl]=transl
		end
	end

--	Some constants and useful unit conversions:
--
-- Time 
	secs_in_day			= 60 * 60 * 24
	secs_in_week			= secs_in_day * 7
	secs_in_yr			= secs_in_day * 365.25
-- Velocity
	lightspeed_mps 	= 300000000
-- Distance
	au_km			= 150000000
	m_per_km		        = 1000
	au_per_ss		        = 100
	auSol2Pluto		= 32.3
	auSol2Earth		= 1
	au_m			        = au_km * m_per_km
	ly_m		        	= secs_in_yr * lightspeed_mps
	ly_km			= ly_m / m_per_km
	uly_km 			= ly_km / 1000000
	ly_au			        = ly_m / au_m

	local colorL 		= { colorL1, colorL2, colorL3, colorL4, colorL5 }
	local pStyle	        	= pointStyle
	local pSize			= pointSize
	local limitLOWER	        = math.min( math.max( show_points_from , 1 ) , 5 )
	local limitUPPER   	= math.max( math.min( show_points_to , 5 ) , limitLOWER )
	local lpMode 		= false
	local txt_title		= _("Lagrange Points v1.1")
	local txt_mode1		= _("Selected")
	local txt_mode2		= _("System")
	local bt_orrery		= _("Orrery")
	local bt_mode 		= _("Mode")
	local bt_verbose   	= _("Verbose")
	local bt_concise   	= _("Concise")
	local bt_marks		= _("Marks")
	local bt_clear		= _("Clear")
	local bt_reset		= _("Clear All")
	-- local metastring	= _("meta-stable")
	local metastring  	= _("unstable")
	local txt_by		= _("by")
	local objectstyle         = {} 		
	local styles                = {}
	styles["clear"]            = 0
	styles["verbose"]        = 1
	styles["concise"]  	= 2
	styles["marks"]   	= 3
	local err			=	_("ERROR").." : "
	local err_nullselection	=	err.._("Please select a valid object.")
	local err_badselection	=	err.._("Selected Object must be a Planet, DwarfPlanet, Moon, or Asteroid.")
	-- local err_badparent	=	err.._("Selected Object must be a Planet, DwarfPlanet, or the Sun.")
	-- local err_badparent	=	err.._("Please select a Planet, DwarfPlanet, or the Sun, ... or change from 'System' to 'Selected' mode.")
	local err_badparent	=	err.._("In 'System' mode the selected Object must be a Planet, DwarfPlanet, or the Sun.")
	width, height              =      celestia:getscreendimension()

	local oldwith, oldheight = 0,0

	local lp_popupheight    = 81
	local vpos_verbose 	= 3
	local vpos_concise 	= 2
	local vpos_mark 	        = 1
	local vpos_orrery 	= 1
	local vpos_mode 	        = 4.25
	local vpos_clear    	= 3
	local vpos_clearall	= 2
	
	local hpos_verbose 	= 2
	local hpos_concise 	= 2
	local hpos_mark 	        = 2
	local hpos_mode 	= 1
	local hpos_clear  	= 1
	local hpos_clearall	= 1

--------------------------------------------------------------------------------------------
--	H O U S E K E E P I N G   F U N C T I O N S
--------------------------------------------------------------------------------------------	

	function	celestia_save_state()
		orig_renderflags                 = celestia:getrenderflags()
		orig_labelflags                   = celestia:getlabelflags()
		orig_orbitflags                    = celestia:getorbitflags()
		orig_amb                           = celestia:getambient()
		orig_maglimit                     = celestia:getfaintestvisible()
		orig_minorbit                      = celestia:getminorbitsize()
		orig_stardistance                = celestia:getstardistancelimit()
		orig_minfeature                  = celestia:getminfeaturesize()
		orig_starstyle                     = celestia:getstarstyle()
		-- orig_time                       = celestia:gettime()
		-- orig_timescale                = celestia:gettimescale()
		orig_galaxylight                 = celestia:getgalaxylightgain()
		orig_fov                            = celestia:getobserver():getfov() 
		orig_locationflags               = celestia:getobserver():getlocationflags()
		orig_overlay                       = celestia:getoverlayelements()
		orig_altazimuthmode           = celestia:getaltazimuthmode()
                --------------------------------------------------------------------------
		-- Comment next line if NOT using Celestia v1.6.0 or later
                --------------------------------------------------------------------------
		orig_textres                       = celestia:gettextureresolution()
                --------------------------------------------------------------------------
		-- Comment next line if NOT using Celestia v1.6.1 or later
                -------------------------------------------------------------------------
		orig_txt_r,orig_txt_g,orig_txt_b = celestia:gettextcolor()
		obs			                     = celestia:getobserver()
		orig_pos		             	     = celestia:geturl(obs)
	end	

	function celestia_restore_state()
	   celestia:setrenderflags(orig_renderflags)
	   celestia:setlabelflags(orig_labelflags)
	   celestia:setorbitflags(orig_orbitflags)
	   celestia:setambient(orig_amb)
	   celestia:setfaintestvisible(orig_maglimit)
	   celestia:setminorbitsize(orig_minorbit)
	   celestia:setstardistancelimit(orig_stardistance)
	   celestia:setminfeaturesize(orig_minfeature)
	   celestia:setstarstyle(orig_starstyle)
	   -- celestia:settime(orig_time + (celestia:getscripttime() / 24 / 3600) )
	   -- celestia:settimescale(orig_timescale)
	   celestia:setgalaxylightgain(orig_galaxylight)
	   celestia:getobserver():setfov(orig_fov)
	   celestia:getobserver():setlocationflags(orig_locationflags)
	   celestia:getobserver():singleview()
	   celestia:setoverlayelements(orig_overlay)
	   celestia:setaltazimuthmode(orig_altazimuthmode)
	   -- Comment next line if NOT using Celestia v1.6.0 or later
	   celestia:settextureresolution(orig_textres)
	   -- Comment next line if NOT using Celestia v1.6.1 or later
	   celestia:settextcolor(orig_txt_r,orig_txt_g,orig_txt_b)
	   
	   return
	end
-----------------------------------------------------------------------------
--	S C R I P T   F U N C T I O N S
-----------------------------------------------------------------------------

	local function switchMode()
		lpMode = not lpMode
	end

	function celestia_set_state() -- modified local/global, necessary for menuLine5Box
		local context = "celestia_set_state(): ";

		celestia:log( context )
		celestia:setorbitflags{
							      Planet = true,
							      Moon = true,
							   -- Asteroid = false,
							   -- Comet = false,
							   -- Spacecraft = false,
							   -- Invisible = false,
							   -- Unknown = false,
							   -- DwarfPlanet = false,     -- Comment line if NOT using Celestia v1.6.0 or later
							   -- MinorMoon = false,       -- Comment line if NOT using Celestia v1.6.0 or later
							   -- Star = false                 -- Comment line if NOT using Celestia v1.6.0 or later
							  }

		celestia:setrenderflags{
						        	-- atmospheres = false,
								-- automag = false,
								-- boundaries = false,
								-- cloudmaps = true,
								-- cloudshadows = false,
								-- comettails = false,
								-- constellations = false,
								-- eclipseshadows = false,
								-- galaxies = true,
								-- grid = false,
								markers = true,
								-- nebulae = false,
								-- nightmaps = false,
								   orbits = true,
								   planets = true,
								-- ringshadows = false,
								-- stars = true,
								-- smoothlines = true,
								-- lightdelay = false,
								-- partialtrajectories = false,
								-- ecliptic = false,       -- Comment line if NOT using Celestia v1.6.0 or later
								-- equatorialgrid = false, -- Comment line if NOT using Celestia v1.6.0 or later
								-- galacticgrid = false,   -- Comment line if NOT using Celestia v1.6.0 or later
								-- eclipticgrid = false,   -- Comment line if NOT using Celestia v1.6.0 or later
								-- horizontalgrid = false  -- Comment line if NOT using Celestia v1.6.0 or later
							   }

		celestia:setlabelflags{
								-- planets = true,
								-- moons = true,
						                -- spacecraft = false,
							        -- asteroids = false,
							        -- comets = false,
							        -- stars = false,
							        -- galaxies = false,
							        -- locations = false,
							        -- constellations = false,
							        -- i18nconstellations = false,
							        -- openclusters = false,
							        -- nebulae = false,
							        -- dwarfplanets = true,      -- Comment line if NOT using Celestia v1.6.0 or later
							        -- minormoons = false,      -- Comment line if NOT using Celestia v1.6.0 or later
							        -- globulars = false            -- Comment line if NOT using Celestia v1.6.0 or later
							  }

		                                                -- celestia:setambient(0.05)
	end
	
	local function mark_lpoints( x , style )
		local i
		local context = "mark_lpoints(): ";
		local vform
		
		celestia:log( context )
		
		for i = limitLOWER,limitUPPER,1 do
			local pL = celestia:find(x:name().."-L"..i)
			if pL:name() == "?" then
				if i > 3 then
					pL = celestia:find(x:name().."-L"..i.." (meta-stable)")
					if pL:name() ~= "?" then
						vform = x:localname().."-L"..i.." ("..metastring..")"
					end
				end
			else
				vform = x:localname().."-L"..i			
			end
			
			if pL:name() ~= "?" then
				pL:unmark()
				if style == styles["verbose"] then
					pL:mark( colorL[i] , pStyle, pSize, 1, vform )
				elseif style == styles["concise"] then
					pL:mark( colorL[i] , pStyle, pSize, 1, "L"..i )
				elseif style == styles["marks"] then
					pL:mark( colorL[i] , pStyle, pSize, 1 )
				end
				objectstyle[ x:name() ] = style
			end
		end
	end -- FUNCTION
	
	local function do_object( sb, style )
		local context = "do_object(): ";

		celestia:log( context )
		celestia:log( "sb:type()="..sb:type() )
	
		-- celestia_set_state()
		if	sb:type() == "null" then
			exception = err_nullselection
			return
		end
		if 		sb:name() ~= "Sol" 
				and sb:type() ~= "planet" 
				and sb:type() ~= "dwarfplanet" 
				and sb:type() ~= "asteroid" 
				and sb:type() ~= "moon" 
				and sb:type() ~= "minormoon" 
				and string.find( sb:name(), "_bary" ) == nil
				and string.find( sb:name(), "-Charon" ) == nil then
			exception = err_badselection 	
			return
		end

		celestia:log( "sb:name() = "..sb:name() )
		local s = string.gsub( sb:name(), "_bary", "" )
		-- celestia:log( "clean _bary: s = "..s )
		s = string.gsub( s, "-Charon", "" )
		-- celestia:log( "clean -Charon: s = "..s )
		sb = celestia:find( s )
		-- celestia:log( "ALL CLEAN sb = "..sb:name() )

		mark_lpoints( sb, style )

	end	-- FUNCTION
	
	
	local function do_system( sb, style )
		local countchildren = 0
		local progress = ""
		local context = "do_system(): ";

		celestia:log( context )
		celestia:log( "sb:type()="..sb:type() )

		-- celestia_set_state()
		if	sb:type() == "null" then
			exception = err_nullselection
			return
		end
		if 		sb:name() ~= "Sol" 
				and sb:type() ~= "planet" 
				and sb:type() ~= "dwarfplanet" 
				-- and sb:type() ~= "asteroid" 
				-- and sb:type() ~= "moon" 
				and string.find( sb:name(), "_bary" ) == nil 
				and string.find( sb:name(), "-Charon" ) == nil then
			exception = err_badparent
			return
		end

		celestia:log( "sb:name() = "..sb:name() )
		local s = string.gsub( sb:name(), "_bary", "" )
		celestia:log( "clean _bary: s = "..s )
		s = string.gsub( s, "-Charon", "" )
		celestia:log( "clean -Charon: s = "..s )
		sb = celestia:find( s )
		celestia:log( "ALL CLEAN sb = "..sb:name() )

		gchildren = sb:getchildren()
		gstyle = style
		gsb = sb
		gchild = 1
		gmarked = 0
		maxchildren = # gchildren
		celestia:log( "sb:type() = "..sb:type() )
		celestia:log( "maxchildren = "..maxchildren )
		if maxchildren > 0 then
			process_children = true
		end
		
	end -- FUNCTION

	local function do_orrery()
		local context = "do_orrery(): ";
		celestia:log( context )

		celestia_set_state()
		sel = celestia:getselection();
		obs = celestia:getobserver()
		if sel:name() == "?" then
			exception = err_nullselection
			celestia:flash( exception, 3 )	
		else
			celestia:log( "sel:name() = "..sel:name() )
			if sel:name() == "Sol" then return end
			now=obs:gettime()
			info = sel:getinfo()
			parent = info.parent
			-- parent=sel:getinfo().parent
			celestia:log( "parent:name() = "..parent:name() )
			parent_pos=parent:getposition(now)
			object_pos=sel:getposition(now)
			distance_vec=parent_pos-object_pos
			distance = (distance_vec:length()*uly_km) / math.tan( obs:getfov() / 2.5 )  
			orr_transition = math.min( math.max( orr_transition, 0 ), 5 )
			obs:gotolonglat(parent, 0, math.rad(90.0), distance, orr_transition, celestia:newvector(0,1,0))
			if ( parent:name() == "Pluto" ) then
				-- celestia:log( "looking for Pluto_bary..." )
				parent = celestia:find("Pluto_bary")
				-- celestia:log( "parent = "..parent:name() )
				if parent:name() == "?" then
					parent = celestia:find("Pluto-Charon")
					-- celestia:log( "looking for Pluto-Charon, parent:name() = "..parent:name() )
				end
			end
			obs:follow(parent)
		end
	end -- FUNCTION	

----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------

LagrangeBox = CXBox:new()
    :init(0, 0, 0, 0)
    :movable(false)
    :attach(screenBox, width, height, 0, 0);

--[[
-- command' button -----------------------------------------------------------

LagrangeSwitch = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0.2,0.8,1,.6})
    :fillcolor(cFrameFill)
    :textfont(smallfont)
    :textcolor({0.2,0.8,1,1})
    :textpos("left")
    :movetext(0, 10)
    :text(_("Lagrange Points"))
    :movable(false)
    :active(true) 
    :attach(menuLine5Box, -93, 0, 0, -11) -- :attach(menuLine5Box, -93, 0, 0, -11)

LagrangeSwitch.Action = function(this)
	LagrangeCmd.Visible = not LagrangeCmd.Visible
	if LagrangeCmd.Visible then 
		celestia_save_state()
		celestia_set_state()
                LagrangeSwitch.Bordercolor={1,0,0,.6}
		LagrangeSwitch.Fillcolor = {cbubordon[1]*2/3, cbubordon[2]*2/3, cbubordon[3]*2/3, cbubordon[4]/2}             
                LagrangeCmd:attach(screenBox, width-LagrangeCmdWidth-35, height-LagrangeCmdHeight-610, 35, 610) 
		LagrangeCmd:orderfront() 
 	else
		celestia_restore_state()
		--LagrangeSwitch.Bordercolor=cbubordoff
                LagrangeSwitch.Bordercolor={0.2,0.8,1,.6}
		LagrangeSwitch.Fillcolor = nil
		resetButton.Action()
                celestia:hide("orbits")   -- добавка 19/3/2014
	end
end
]]
-- frame ----------------------------------------------------------------------------
LagrangeCmd = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0.2,0.8,1,.6})
    :fillcolor(lp_ccmdbofill)
    :movable(true)
	:clickable(false)
	:visible(false)
    :attach(screenBox, width, height, 0, 0)
----------------------------------------------
LagrangeCmd.Customdraw = function(this)

            if Lagrange_firstTickDone then
               if not(LagrangeCloseButtonDone) then
                   LagrangeCloseButtonDone = true;
                   makeCloseButton(LagrangeCloseButton, LagrangeCmd, LagrangeSwitch);
               end
            else
                Lagrange_firstTickDone = true;         
            end 

            if newwidth ~= width or newheight ~= height then
                    newwidth, newheight = width, height; 
               if height>=768  then              
                    LagrangeCmd:attach(screenBox, width-LagrangeCmdWidth-35, height-LagrangeCmdHeight-610, 35, 610) 
               else
                    LagrangeCmd:attach(screenBox, width-LagrangeCmdWidth-200, height-LagrangeCmdHeight-3, 200, 3)  
               end                   
            end

	textlayout:setfont(normalfont);
        textlayout:setfontcolor({0,1,0,0.8});             
        textlayout:setpos(this.lb+30, this.tb - 13);

        textlayout:println(txt_title);

end

-- ===================
-- Подключение команд 
-- =================== 
modeButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)      
    :fillcolor(lp_cbufilloff)
    :textfont(normalfont)
    :textcolor(color1)
    :movable(false)
    :active(true) 
    :attach(LagrangeCmd, 4, 4+17*(gg-1), 95, 3+17*1)
--------------------------------------	
modeButton.Action =	function()
	switchMode()
end
--------------------------------------
modeButton.Customdraw = function(this)
	textlayout:setfont(normalfont);
	textlayout:setpos(this.lb+20, this.tb - 10);

	textlayout:setfont(normalfont);
	if lpMode then	
		-- System mode
		if lang == "nl" then 
			textlayout:setpos(this.lb+20, this.tb - 10)
		elseif lang == "ru" then 
			textlayout:setpos(this.lb+15, this.tb - 10)
		else
			textlayout:setpos(this.lb+8, this.tb - 10);
		end
		textlayout:setfontcolor({0, 1, 0, 0.99});	-- green
                local obs = celestia:getobserver();
                local obj = celestia:find("Sol");
                celestia:select(obj);
                obs:follow(obj);          
                obs:center(obj); 
		textlayout:println(txt_mode2);
	else
		-- Selected mode
		if lang == "en" then 
			textlayout:setpos(this.lb+7, this.tb - 10)
		elseif lang == "nl" then 
			textlayout:setpos(this.lb+7, this.tb - 10)
		elseif lang == "ru" then 
			textlayout:setpos(this.lb+22, this.tb - 10)
		else
			textlayout:setpos(this.lb+2, this.tb - 10);
		end
		textlayout:setfontcolor({1, 1, 0, 0.99});	-- yellow
		textlayout:println(txt_mode1);
	end
end
-- =================== 
labelButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)      
    :fillcolor(lp_cbufilloff)
    :textfont(normalfont)
    :textcolor(color1)
    :textpos("center")    
    :movetext(0, 8)
    :text(bt_verbose)     -- = _("Verbose") = "Подробно"; 
    :movable(false)
    :active(true) 
    :attach(LagrangeCmd, 4, 4+17*(gg-2), 95, 3+17*2)
--------------------------------------
labelButton.Action = function()
	local sel = celestia:getselection();
	local context = "labelButton.Action (): ";
	celestia:log( context )

	if lpMode then
		do_system( sel, styles["verbose"] )
	else
		do_object( sel, styles["verbose"] )
	end
end
----------------------------------------
labelButton.Customdraw = function()
	local context = "labelButton.Customdraw(): ";

	if exception ~= nil then
		celestia:flash( exception, 3 )	
		exception = nil
	elseif process_children then
		local x = gchildren[gchild]
		local y = celestia:find(x:name())
		-- skip numerous objects which aren't Planets, DwarfPlanets, or Moons,
                -- пропустить многочисленные объекты, которые не являются Планетами, Малыми планетами или Луны, 
		-- and skip Selden's Hale Telescope dummy objects...

		if 	
			(y:type() ~= "asteroid") and 
			(y:type() ~= "comet") and 
			(y:type() ~= "invisible") and 
			(y:type() ~= "spacecraft") and 
			(y:type() ~= "component") and 
			(x:name() ~= "?") and
			(x:name() ~= "Neptune-noao-1999") and
			(x:name() ~= "Neptune-ao-1999") and
			(x:name() ~= "Neptune-ao-2006") and
			(x:name() ~= "Uranus-ao-2006") and
			(string.match ( x:name() , ".*%-L%d$") == nil ) 
		then
			mark_lpoints( x, gstyle )
			gmarked = gmarked + 1
			celestia:log( context.."(".. gmarked..") : "..x:name() )
		end 

		gchild = gchild + 1
		if gchild > maxchildren then
			process_children = false
		end
	end
end
-- ===================
conciseButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)      
    :fillcolor(lp_cbufilloff)
    :textfont(normalfont)
    :textcolor(color1)
    :textpos("center")    
    :movetext(0, 8)
    :text(bt_concise)                      -- = _("Concise") = "Кратко"; 
    :movable(false)
    :active(true) 
    :attach(LagrangeCmd, 4, 4+17*(gg-3), 95, 3+17*3)

conciseButton.Action = function()
	local sel = celestia:getselection();

	if lpMode then
		do_system( sel, styles["concise"] )
	else
		do_object( sel, styles["concise"] )
	end
end
-- ===================
markButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)      
    :fillcolor(lp_cbufilloff)
    :textfont(normalfont)
    :textcolor(color1)
    :textpos("center")    
    :movetext(0, 8)
    :text(bt_marks)                     -- = _("Marks") = "Метки";
    :movable(false)
    :active(true) 
    :attach(LagrangeCmd, 4, 4+17*(gg-4), 95, 3+17*4)
--------------------------------------
markButton.Action = function()
	local sel = celestia:getselection();		

	if lpMode then
		do_system( sel, styles["marks"] )
	else
		do_object( sel, styles["marks"] )
	end
end
-- =================== 
clearButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)      
    :fillcolor(lp_cbufilloff)
    :textfont(normalfont)
    :textcolor(color1)
    :textpos("center")    
    :movetext(0, 8)
    :text(bt_clear)        -- = _("Clear") = "Очистить"; 
    :movable(false)
    :active(true) 
    :attach(LagrangeCmd, 95, 4+17*(gg-1), 4, 3+17*1)
---------------------------------------
clearButton.Action = function()
	local sel = celestia:getselection();

	if lpMode then
		do_system( sel, styles["clear"] )
	else
		do_object( sel, styles["clear"] )
	end
end 
--==================== 
resetButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)      
    :fillcolor(lp_cbufilloff)
    :textfont(normalfont)
    :textcolor(color1)
    :textpos("center")    
    :movetext(0, 8)
    :text(bt_reset)                  -- = _("Clear All") = "Очистить все"; 
    :movable(false)
    :active(true) 
    :attach(LagrangeCmd, 95, 4+17*(gg-2), 4, 3+17*2)
--------------------------------------
resetButton.Action = function()
	celestia:unmarkall()
end
-- ===================
orreryButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)      
    :fillcolor(lp_cbufilloff)
    :textfont(normalfont)
    :textcolor(color1)
    :textpos("center")    
    :movetext(0, 8)
    :text(bt_orrery)                 -- = _("Orrery")  = "Обзор"
    :movable(false)
    :active(true) 
    :attach(LagrangeCmd, 95, 4+17*(gg-3), 4, 3+17*3)
--------------------------------------
orreryButton.Action  = function()
	local context = "orreryButton.Action (): ";
	celestia:log( context )		
	do_orrery()
end
-- =================== 
LagrangeByBox = CXBox:new()
    :init(0, 0, 0, 0)
    :textfont(smallfont)
    :textcolor(color0)
    :textpos("center")    
    :movetext(0, 8)
    :text(_("by Phil Batchelor"))
    :movable(false)
    :active(false) 
    :attach(LagrangeCmd, 95, 4+17*(gg-4), 4, 3+17*4)

-- ===================

