---------------------------------------------------
-- menuLine1Box.lua
-- by Gironde 08-04-2017
-- edit Patrick Ziegler a.k.a. Gironde 10-04-2017
-- edit by Lygin Gennady a.k.a. Croc 19.08.2018
---------------------------------------------------

--require "locale"
currentTimedisplay = true -- global variable for also using in panel7

local timescale1_str = "" -- by Gironde
local timescale2_str = "" -- by Gironde

------------------hide Celestia clock-----------------
actual_overlaytab = celestia:getoverlayelements() -- create and initialize table first for save overlayelements
	overlaytab = {} -- new table
	overlaytab.Time = false 	-- modify display time overlayelement
	
celestia:setoverlayelements(overlaytab) -- set overlayelements
------------------------------------------------------

menuLine1Box= CXBox:new() -- TimeStopBox = CXBox:new()
	:init(0, 0, 0, 0)
	:movable(false)
	:attach(screenBox, 0, 0, 0, 0) -- all the screenbox

menuLine1Button = CXBox:new() -- button of menuLine1 : inactif here
	:init(0, 0, 0, 0)
	:bordercolor({0.2,0.8,1,.6}) 
	:fillcolor(cFrameFill)
	:attach(menuLine1Box,-187-14,0-7,0,-32+7)  

menuLine1Button.Customdraw = function(this)
	menuLine1Button.Bordercolor = {0.2,0.8,1,.6}
end

--==================================================================
-- timeFrameUpperRigth
-- coded by Gironde 31-08-2018 (cloned and adapted from panel7)
-- this code display an other clock (the same that in panel7)
--==================================================================

----------------------------------------------
-- Define local functions
----------------------------------------------
local setDateColorsOff = function()
	daycolor = cbutextoff
	monthcolor = cbutextoff
	yearcolor = cbutextoff
	hourcolor = cbutextoff
	minutecolor = cbutextoff
	secondcolor = cbutextoff
	centiseccolor = color2 -- added 14/08/2018 by Gironde
	timezonecolor = cbutextoff
end

local setCurrentTime = function()
	local UTCtime = os.date("!*t") -- returns the converted time to universal time coordinates
	celestia:settime(celestia:utctotdb(UTCtime.year, UTCtime.month, UTCtime.day, UTCtime.hour, UTCtime.min, UTCtime.sec))
end

local setTime = function()
	celestia:settime(celestia:utctotdb(date.year, date.month, date.day, date.hour, date.minute, date.seconds))
end

----------------------------------------------
-- Set initial values
----------------------------------------------
-- Define a limit value for timescale.
-- This shouldn't be necessary but timerate strangely
-- keeps on increasing when time reaches limit values...
limitTimerate = 1e15

-- Define max/min values for time to avoid bugs with Julian days...
maxTime = 730486721060.00073 -- 2000000000 Jan 01 12:00:00 UTC
minTime = -730498278941.99951 -- -2000000000 Jan 01 12:00:00 UTC

setDateColorsOff()
local daycolor = cbutexton
local toset = "day"
local previouscolor = color4 -- added 13/08/2018 by Gironde
local previous100color = color0 -- added 13/08/2018 by Gironde

-- Get displayUTCtime value from 'config.lua'.
displayUTCtime = not(show_local_time) 	-- global parameter, show_local_time is in config.lua
										-- displayUTCtime is inverse of config parameter
-- Get timezone_cfg value from 'config.lua'.
timezone_cfg = time_zone -- global parameter

timescale = celestia:gettimescale() -- global parameter
paused = celestia:ispaused() -- global parameter

-- =================================================
-- Размеры панели коррекции времени: ширина / высота размер пиктограмм
-- =================================================

-- =================================================
-- Set up and Draw
-- =================================================
------------------------------------------------------------------------
-- Общая рамка для Машины Времени (для часов и кнопок)
------------------------------------------------------------------------

--------------------------------
-- timeFrameUpperRight : replace celestia clock
--------------------------------
timeFrameUpperRight = CXBox:new()
	:init(0, 0, 0, 0)
	:movable(false)
	:attach(menuLine1Button, 2, 20, 2, 0)

timeFrameUpperRight.Customdraw =
	function(this)
	
		local displayUTCtime = true -- force the UTC mode

		celestia:setoverlayelements(overlaytab)
		local time = celestia:gettime()
		local timescale = celestia:gettimescale()
		-- Fix max/min time values.
		if time <= minTime then
			celestia:settime(minTime)
			celestia:settimescale(0)
		elseif time >= maxTime then
			celestia:settime(maxTime)
			celestia:settimescale(0)
		end
		----------------------------------------------------------
		-- Convert Celestia's internal TDB time to UTC date table.
		-- Преобразование внутреннего времени TDB Celestia в UTC...
		----------------------------------------------------------
		date = celestia:tdbtoutc(time)
		----------------------------------------------------------
		-- Get OS time in seconds from the UTC date table.
		-- Получить OS время в секундах, из UTC таблицы "дата" .
		----------------------------------------------------------

		local systime = os.time{year=date.year, month=date.month, day=date.day, hour=date.hour, min=date.minute, sec=date.seconds}

		if systime then
			-- Get the local date table from OS.
			local syslocdate  = os.date("*t", systime + 3600)
			-- Get the UTC date from OS.
			local sysUTCdate = os.date("!*t", systime + 3600)
			-- Compute the system locale Time Zone.
			local daydiff = syslocdate.day - sysUTCdate.day
			local hourdiff

			if daydiff == 0 then
				hourdiff = 0
			elseif daydiff > 1 or daydiff == -1 then
				hourdiff = -24
			else
				hourdiff = 24
			end

			timezone = hourdiff + syslocdate.hour - sysUTCdate.hour + (syslocdate.min - sysUTCdate.min) / 60

			---------------------------------------------------------------------------------------------------
			-- HACK: Get the local date table from the 'modified' TDB time using Celestia's 'tdbtoutc' function.
			-- HACK: Получить локальную таблицу дату из "модифицированной" TDB времени с использованием функции Celestia  "tdbtoutc".
			---------------------------------------------------------------------------------------------------

			locdate = celestia:tdbtoutc(time + timezone/24)
		end
		-----------------------------------------------------------
		-- Set Time Zone string
		-- Установить строку часовой пояс
		-----------------------------------------------------------

		if displayUTCtime or not(systime) then -- if displayUTCtime is true
			timezone_str = "   UTC"
			day_str = math.abs(date.day) 
            month_str = monthAbbr[math.abs(date.month)] 
            year_str = date.year
			hour_str = format:twodigits(math.abs(date.hour)) 
            min_str = format:twodigits(math.abs(date.minute))
			sec_str = format:twodigits(math.floor(math.abs(date.seconds)))
			centisec_str = format:twodigits((math.abs(date.seconds) - math.floor(math.abs(date.seconds)))*100)

		elseif type(time_zone) == "number" then -- if displayUTCtime is false then GMT+number
			------------------------------------------------------
			-- A Time Zone value has been set in config.lua.
			-- Значение часового пояса было установлено в config.lua.
			------------------------------------------------------

			date_cfg = celestia:tdbtoutc(time + timezone_cfg/24)
			timezone_str = "GMT"..string.format("%+0.0f", math_sign(timezone_cfg) * math.floor(math.abs(timezone_cfg)))
			if math.floor(timezone_cfg) ~= timezone_cfg then
				----------------------------------------------
				-- Timezone is an non-integer number of hours.
				-- Часовой пояс не целое количество часов
				----------------------------------------------
				timezone_str = timezone_str.._("h")..(math.abs(timezone_cfg) - math.floor(math.abs(timezone_cfg))) * 60
			end
			
			year_str = date_cfg.year
			day_str = date_cfg.day 
			month_str = monthAbbr[date_cfg.month] 
			hour_str = format:twodigits(date_cfg.hour) 
			min_str = format:twodigits(date_cfg.minute) 
			sec_str = format:twodigits(math.floor(date_cfg.seconds))
			centisec_str = format:twodigits(math.floor((date_cfg.seconds - (math.floor(math.abs(date_cfg.seconds))))*100))
			
		else
			-----------------------------------
			-- Use system locale Time Zone.
			-- Использование языка системы часовых поясов.
			-----------------------------------

			timezone_str = "GMT"..string.format("%+0.0f", math_sign(timezone) * math.floor(math.abs(timezone)))
			if math.floor(timezone) ~= timezone then
				-------------------------------------------------
				-- Timezone is an non-integer number of hours.
				-- Часовой пояс не целое количество часов
				-------------------------------------------------
				timezone_str = timezone_str.._("h")..(math.abs(timezone)-math.floor(math.abs(timezone)))*60
			end
			
			year_str = locdate.year
			day_str = locdate.day 
			month_str = monthAbbr[locdate.month] 
			hour_str = format:twodigits(locdate.hour) 
			min_str = format:twodigits(locdate.minute) 
			sec_str = format:twodigits(math.floor(locdate.seconds))
			centisec_str = format:twodigits(math.floor((locdate.seconds - (math.floor(math.abs(locdate.seconds))))*100))
		end
	   ---------------------------------------------------------------
	   -- Определение дня недели:
	   ---------------------------------------------------------------
		local datetime = os.time({ year=date.year, month = date.month, day = date.day, hour = date.hour+4, minute = date.minute})
		local weekday = os.date("%w",datetime)

		-------------------------------------------------------------------------------------
		-- Set Date string. Use date format from the locale/lang/lang.lua file.
		-- Определить дату (строку). Используйте формат даты из locale/ Lang / lang.lua файл.
		-------------------------------------------------------------------------------------

		if date_format == "year/month/day" then
			------------------------
			-- Big endian format.
			-- Формат ГГГГ-ММ-ДД
			------------------------
			date_str = year_str..' '..month_str..' '..day_str
		elseif date_format == "month/day/year" then
			--------------------------
			-- Middle endian format.
			-- Формат ММ-ДД-ГГГГ
			--------------------------
			date_str = month_str..' '..day_str..' '..year_str
		else
			----------------------------------
			-- Little endian format (default).
			-- Формат ДД-ММ-ГГГГ
			----------------------------------

			date_str = day_str..' '..month_str..' '..year_str
		end
		---------------------------------------------------------------------------------
		-- Set Time string.
		-- Определить время (строку).
		---------------------------------------------------------------------------------

		time_str = hour_str..': '..min_str..': '..sec_str

		---------------------------------------------------------------------------------
		-- Display Date, Time, Timezone.
		-- Отобразить дату, время, часовой пояс.
		---------------------------------------------------------------------------------

		textlayout:setfont(normalfont)
		textlayout:setlinespacing(normalfont:getheight())
		------------------------------------------------ date_str
		
		textlayout:setfontcolor({0.2,0.8,1,1})
		textlayout:setpos(this.lb+5, this.tb-13)
		textlayout:println(date_str)
		
		------------------------------------------------ time_str
		
		textlayout:setpos(this.lb+110, this.tb-13)

		if year_str <100000 then		
		    textlayout:println(time_str)
		end
		
		------------------------------------------------- timezone_str 
		
		textlayout:setpos(this.lb+160, this.tb-13)
		textlayout:println(timezone_str)
		
		------------------------------------------------ weekday
		textlayout:setfontcolor({1,0,0,0.9})
		textlayout:setpos(this.lb+5, this.tb-23-3)
		
		if weekday == "0" then
			textlayout:println (_"Sunday") -- Dimanche, Воскресенье
		elseif weekday == "1" then
			textlayout:println (_"Monday") -- Lundi, Понедельник ******* 11 characters
		elseif weekday == "2" then
			textlayout:println (_"Tuesday") -- Mardi, Вторник
		elseif weekday == "3" then
			textlayout:println (_"Wednesday") -- Mercredi, Среда
		elseif weekday == "4" then
			textlayout:println (_"Thursday") -- Jeudi, Четверг
		elseif weekday == "5" then
			textlayout:println (_"Friday") -- Vendredi, Пятница
		elseif weekday == "6" then
			textlayout:println (_"Saturday") -- Samedi, Суббота
		end

		if celestia:ispaused() then

			---------------------------------------------------
			-- Time is paused
			---------------------------------------------------
			timescale_str = ""
			textlayout:setfontcolor(ctextalert)
			textlayout:setpos(this.lb+30+normalfont:getwidth(_("Rate:"))+35, this.tb-26) 
			textlayout:println(_("pause"))
			textlayout:setfontcolor(ctext)
		elseif math.abs(timescale) <= 1e-15 then -- considered 0		
	
			---------------------------------------------------
			-- Time is stopped.
			---------------------------------------------------

			timescale_str = _("time stop")
		else
			timescale_str = string.format("%g", timescale).."x"			
		end

		------------------------------------------------------
		-- Display Timerate
		-- Отобразить Множитель времени. --  Rate
		------------------------------------------------------
		textlayout:setfontcolor(cbutextoff)		
        textlayout:setpos(this.lb+90, this.tb-26)		
        textlayout:println(timescale_str) 
		------------------------------------------------------
end -- end of Customdraw

-- ===========================================================================================================================

