--======================================
-- addsBox.lua : List of addons: special renders and celx
-- Author - Vincent Giangiulio a.k.a Vincent
-- Edit by Joёl a.k.a Jogad 
-- Integration by Lygin Gennady a.k.a Croc 21.12.2013 и  28.08.2016
-- Edit by Patrick Ziegler a.k.a. Gironde 07-04-2017
--======================================
require "config"  -- new Jogad (здесь и далее 28.08.2016)
require "locale";
require "closeButton";
require "scrollButton";

----------------------------------------------
-- Set initial values
----------------------------------------------
local newwidth1, newheight1;
local Adds_list_nlines = Adds_list_nlines or 4;
local nAdds = table.getn(adds);
local nAddsLinesMax = math.max(4, math.min(Adds_list_nlines, nAdds));
local AddsFrameWidth, AddsFrameHeight = 250, 50 + (20 * nAddsLinesMax);
local AddsScrollbarFrameW, AddsScrollbarFrameH = 9, AddsFrameHeight  - 70;
local nAddsLines = nAdds;
 		
local AddsScroll = 0;
local AddsScrollCoeff = 6;
local AddsScrollbarBoxH = 0;
local addLeftOffset = 10;

local addBox = {};
local addCheckBox = {};
local addVisible = {};
local addLabelColor = {};
local adds_firstTickDone = false 
local closeButtonDone = false

celestia:setlabelflags{locations = true}

-- function used in ViDiBaBox, renderBox and addsBox
local setCheckBoxText = function(box, k, flag)
   if flag then
        box[k].Text = "v";
    else
        box[k].Text = "";
    end
end
----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------

-- command' button --------------------------------------------------------

addsBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor({0.2,0.8,1,.6})
    :fillcolor(cFrameFill)
    :movable(false)
    :attach(screenBox, width, height, 0, 0)


-- frame ------------------------------------------------------------------------------------

addsFrame = CXBox:new()
    :init(0, 0, 0, 0)
    :fillcolor(csetbofill)
    :bordercolor(cbubordoff)
    :movable(true)
    :visible(false)
    :attach(screenBox, (width/2-AddsFrameWidth)-3, height-AddsFrameHeight-2, width/2+3, 2)

addsFrame.Customdraw = function(this)
            textlayout:setfont(normalfont);
            textlayout:setfontcolor({0,1,0,0.8});  
            textlayout:setpos(this.lb+addLeftOffset, this.tb-20);
            textlayout:println(_("ADDONS visibility:"));

		if adds_firstTickDone then
            if not(closeButtonDone) then
                closeButtonDone = true;
                makeCloseButton(addsCloseButton, addsFrame, setAddsButton);
            end
        else
            adds_firstTickDone = true;
        end
        if newwidth1 ~= width or newheight1 ~= height then
            newwidth1, newheight1 = width, height
            addsFrame:attach(screenBox, (width/2-AddsFrameWidth)-3, height-AddsFrameHeight-10, width/2+3, 10)
	    end	
    end;
---------------------------------------

local makeAddBox = function(k, v, addleftOffset, color) -- edited by Gironde 13-04-2017

addBox[k] = CXBox:new()
        :init(0, 0, 0, 0)
        --:bordercolor(cbubordoff)
        --:fillcolor({1,1,0,.2})
        :textfont(normalfont)
        :textcolor(color)
        :movetext(0, 8)    
        :text(_(v))
        :movable(false)
        :attach(addsFrame, addLeftOffset+15, AddsFrameHeight-25-(20*k), AddsFrameWidth-addLeftOffset-100, 14+(20*k)) -- edited by Gironde 13-04-2017
end

local makeAddCheckBox = function(k, addleftOffset)
    addCheckBox[k] = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(checkBcolor)  -- :bordercolor(cbubordoff)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(addsFrame, addLeftOffset, AddsFrameHeight-25-(20*k), AddsFrameWidth-addLeftOffset-11, 14+(20*k)) -- edited by Gironde 13-04-2017
end

for k, v in ipairs(adds) do

    if pcall( function() require (v) end ) then
        require(v)

        addLabelColor[k] = cbutextoff;
        if _G[v].labelcolor then
            addLabelColor[k] = _G[v].labelcolor;
        end

        makeAddBox(k, string.gsub(v, "_", " "), addLeftOffset, addLabelColor[k]);
        makeAddCheckBox(k, addLeftOffset);

        local vv, kk = v, k;

        if _G[v].objects then
            addVisible[k] = celestia:find(_G[v].objects[1]):visible();
        else
            addVisible[k] = false;
        end

        addBox[k].Visible = k <= nAddsLinesMax;
        addCheckBox[k].Visible = k <= nAddsLinesMax;

        addCheckBox[k].Customdraw =
            function(this)
                setCheckBoxText(addCheckBox, kk, addVisible[k + AddsScroll]); -- setAddCheckBoxText or setCheckBox2Text
                addBox[k].Text = _(string.gsub(adds[k + AddsScroll], "_", " "));
                addBox[k].Textcolor = addLabelColor[k + AddsScroll];
            end

            addCheckBox[k].Action = (function()
                return
                    function()
                        local vAddsScroll = adds[k + AddsScroll];
                        addVisible[k + AddsScroll] = not(addVisible[k + AddsScroll]);
                        if _G[vAddsScroll].objects then
                            for n, obj in pairs(_G[vAddsScroll].objects) do
                                local addObj = celestia:find(obj);
                                addObj:setvisible(not addObj:visible());
                            end
                        end

                        if _G[vAddsScroll].locationtypes then
                            for n, loctype in pairs(_G[vAddsScroll].locationtypes) do
                                addLocType = string.lower(loctype);
                                local obs = celestia:getobserver();
                                local locFlags = obs:getlocationflags();
                                if locFlags[addLocType] ~= addVisible[k + AddsScroll] then
                                    locFlags[addLocType] = addVisible[k + AddsScroll];
                                    obs:setlocationflags(locFlags);
                                end
                            end
                        end

                        if _G[vAddsScroll].script then
                            local scriptfile = "../adds/"..vAddsScroll.."/".._G[vAddsScroll].script;
                            celestia:runscript(scriptfile);
                            if _G[vAddsScroll].objects then
                                addVisible[k + AddsScroll] = celestia:find(_G[vAddsScroll].objects[1]):visible();
                                setCheckBoxText(addCheckBox, kk, addVisible[k + AddsScroll]); -- setAddCheckBoxText or setCheckBox2Text
                            end
                        end
                    end
                end) ();
    end
end

----------------------------------------------------------------
AddsScrollbarFrame = CXBox:new()
    :bordercolor(cbubordoff)  -- réactivé
    --:fillcolor({1, 0, 0, 1})
    :visible(nAddsLines > nAddsLinesMax)
    :attach(addsFrame, AddsFrameWidth-AddsScrollbarFrameW-3, AddsFrameHeight-40-AddsScrollbarFrameH, 3, 40);

AddsScrollbarFrame.Customdraw =
    function(this)
        -- We add the scrollButton on the second tick
        -- when bounds are getting updated.
        if AddsScroll_firstTickDone then
            if not(AddsScrollButtonDone) then
                AddsScrollButtonDone = true;
                makeScrollButton(AddsScrollUpButton, AddsScrollDownButton, AddsScrollbarFrame);
            end
        else
            AddsScroll_firstTickDone = true;
        end

        local lb3,bb3,rb3,tb3 =AddsScrollbarFrame:bounds();
        local lb4,bb4,rb4,tb4 = AddsScrollbarBox:bounds();
        AddsScroll = math.ceil((tb3-tb4-3)/AddsScrollCoeff); -- math_round
        AddsScrollbarFrame:attach(addsFrame, AddsFrameWidth-AddsScrollbarFrameW-3, AddsFrameHeight-40-AddsScrollbarFrameH, 3, 40);
    end

AddsScrollbarBox = CXBox:new() -- cursor of scrollbar
    :bordercolor(checkBcolor) -- cbubordoff
    :fillcolor(cscrollfill)
    :visible(true)
    :movable(true)
    :attach(AddsScrollbarFrame, 0, (nAddsLines-nAddsLinesMax)*AddsScrollCoeff, 0, 3)
