---------------------------------------------------------------------------
-- OverviewOrbitsBox.lua (tool from Lagrange Points Tool v1.1)
-- by Phil Batchelor a.k.a chuft-captain
-- Edit by Lygin Gennady a.k.a Croc 08.03.2015
---------------------------------------------------------------------------
	require "locale"
	require "lp_config"

--	Some constants and useful unit conversions:
--
-- Time 
	secs_in_day			= 60 * 60 * 24
	secs_in_week			= secs_in_day * 7
	secs_in_yr		        	= secs_in_day * 365.25
-- Velocity
	lightspeed_mps 	= 300000000
-- Distance
	au_km		= 150000000
	m_per_km		= 1000
	au_per_ss		= 100
	auSol2Pluto	= 32.3
	auSol2Earth	= 1
        au_m			= au_km * m_per_km
	ly_m			= secs_in_yr * lightspeed_mps
	ly_km		= ly_m / m_per_km
	uly_km 		= ly_km / 1000000
--	ly_au			= ly_m / au_m

-----------------------------------------------------------------------------
--	S C R I P T   F U N C T I O N S
-----------------------------------------------------------------------------
	local function celestia_set_state()
		local context = "celestia_set_state(): ";
		celestia:log( context )
		celestia:setorbitflags{ Planet = true, Moon = true,}
		celestia:setrenderflags{markers = true,	orbits = true, planets = true, }
	end


	local function do_Orrery()
		local context = "do_Orrery(): ";
		celestia:log( context )

		celestia_set_state()
		sel = celestia:getselection();
		obs = celestia:getobserver()
		if sel:name() == "?" then
			exception = err_nullselection
			celestia:flash( exception, 3 )	
		else
			celestia:log( "sel:name() = "..sel:name() )
			if sel:name() == "Sol" then return end
			now=obs:gettime()
			info = sel:getinfo()
			parent = info.parent
			celestia:log( "parent:name() = "..parent:name() )
			parent_pos=parent:getposition(now)
			object_pos=sel:getposition(now)
			distance_vec=parent_pos-object_pos
			distance = (distance_vec:length()*uly_km) / math.tan( obs:getfov() / 2.5 )  
			orr_transition = math.min( math.max( orr_transition, 0 ), 5 )
			obs:gotolonglat(parent, 0, math.rad(90.0), distance, orr_transition, celestia:newvector(0,1,0))
			if ( parent:name() == "Pluto" ) then
				-- celestia:log( "looking for Pluto_bary..." )
				parent = celestia:find("Pluto_bary")
				-- celestia:log( "parent = "..parent:name() )
				if parent:name() == "?" then
					parent = celestia:find("Pluto-Charon")
					-- celestia:log( "looking for Pluto-Charon, parent:name() = "..parent:name() )
				end
			end
			obs:follow(parent)
		end
	end -- FUNCTION	

-- =======================
-- Set up and Draw the boxes
----------------------------------------------
OverviewOrbitsBox = CXBox:new()    
    :fillcolor(cFrameFill)
    :init(0, 0, 0, 0)
    :movable(false)
    :attach(screenBox, width, height, 0, 0);

OverviewOrbitsButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0,0.5,0.5,.5})
    :fillcolor(cFrameFill)
    :textfont(smallfont)
    :textcolor({0.2,0.8,1,1})
    :textpos("center")
    :movetext(-3, 8)  
    :text(_("OOO"))
    :movable(false)
    :active(true) 
    :attach(OverviewOrbitsBox,0,0,0,0) 

OverviewOrbitsCheck = CXBox:new()
        :init(0, 0, 0, 0)
        --:bordercolor()
        --:fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor({1,0,0,1})
        :textpos("left")
        :movetext(-32, 9)
        :text("")
        :movable(false)
        :active(true)  
        :attach(OverviewOrbitsBox, 20, 0, 0, 0)

OverviewOrbitsButton.Action = (function()
        return
            function()                 
                if OverviewOrbitsCheck.Text == "" then
                   OverviewOrbitsCheck.Text = "o"; 

		orig_renderflags                 = celestia:getrenderflags()
		orig_labelflags                  = celestia:getlabelflags()
		orig_orbitflags                  = celestia:getorbitflags()
		orig_amb                         = celestia:getambient()
		orig_maglimit                    = celestia:getfaintestvisible()
		orig_minorbit                    = celestia:getminorbitsize()
		orig_stardistance              = celestia:getstardistancelimit()
		orig_minfeature                = celestia:getminfeaturesize()
		orig_starstyle                   = celestia:getstarstyle()
		orig_galaxylight                 = celestia:getgalaxylightgain()
		orig_fov                            = celestia:getobserver():getfov() 
		orig_locationflags               = celestia:getobserver():getlocationflags()
		orig_overlay                       = celestia:getoverlayelements()
		orig_altazimuthmode           = celestia:getaltazimuthmode()
                --------------------------------------------------------------------------
		-- Comment next line if NOT using Celestia v1.6.0 or later
                --------------------------------------------------------------------------
		orig_textres                     = celestia:gettextureresolution()
                --------------------------------------------------------------------------
		-- Comment next line if NOT using Celestia v1.6.1 or later
                -------------------------------------------------------------------------
		orig_txt_r,orig_txt_g,orig_txt_b = celestia:gettextcolor()
		obs			                     = celestia:getobserver()
		orig_pos		             	     = celestia:geturl(obs)

-- ===============================================
	local context = "OrreryButton.Action (): ";
	celestia:log( context )		
	do_Orrery()
-- ===============================================

           else  
                    OverviewOrbitsCheck.Text = "";  

	   celestia:setrenderflags(orig_renderflags)
	   celestia:setlabelflags(orig_labelflags)
	   celestia:setorbitflags(orig_orbitflags)
	   celestia:setambient(orig_amb)
	   celestia:setfaintestvisible(orig_maglimit)
	   celestia:setminorbitsize(orig_minorbit)
	   celestia:setstardistancelimit(orig_stardistance)
	   celestia:setminfeaturesize(orig_minfeature)
	   celestia:setstarstyle(orig_starstyle)
	   celestia:setgalaxylightgain(orig_galaxylight)
	   celestia:getobserver():setfov(orig_fov)
	   celestia:getobserver():setlocationflags(orig_locationflags)
	   celestia:getobserver():singleview()
	   celestia:setoverlayelements(orig_overlay)
	   celestia:setaltazimuthmode(orig_altazimuthmode)
	   -- Comment next line if NOT using Celestia v1.6.0 or later
	   celestia:settextureresolution(orig_textres)

            local obs = celestia:getobserver();
            local obj = celestia:find("Earth");
            celestia:select(obj);
            obs:follow(obj);          
            obs:goto(obj); 

                end
            end
        end) ();
