-- =================================
-- compassBox.lua
-- Author of panel - Gennady Lygin a.k.a Croc
-- Author of tools - Vincent Giangiulio a.k.a Vincent
-- Integration by Gennady Lygin a.k.a Croc 16.05.2014, 11.08.2018
-- Edit by Gennady Lygin a.k.a Croc & Patrick Ziegler a.k.a. Gironde 03-04-2017
-- =================================

-- =================================
-- compassBox.lua
-- =================================
require "locale"
require "closeButton"
----------------------------------------------
-- Define the local compass image.
----------------------------------------------
compImageFilename = "../locale/"..lang.."/images/compass_"..lang..".png"
compassImage = celestia:loadtexture(compImageFilename)

if not(compassImage) then
    compImageFilename = "../images/compass.png"
    compassImage = celestia:loadtexture(compImageFilename)
end

----------------------------------------------
-- Define local functions
----------------------------------------------

local rotateCompass = function (x,y,w)
    -- Rotate the compass.
    x = x - (lb + compassW / 2)
    y = y - (bb + compassH / 2)
    local x2 = x * math.cos(w) - y * math.sin(w)
    local y2 = y * math.cos(w) + x * math.sin(w)

    -- A translation is also needed to make the compass rotate about its center.
    x2 = x2 + (lb + compassW / 2)
    y2 = y2 + (bb + compassH / 2)
    return x2, y2;
end

local drawCompass = function(angle)
    -- Draw the compass.
    local angle_rad = angle * degToRad;
    lb,bb,rb,tb = compassFrameBox:bounds()
    local xlb, ylb = rotateCompass(lb, bb, angle_rad)
    local xrb, yrb = rotateCompass(rb, bb, angle_rad)
    local xrt, yrt = rotateCompass(rb, tb, angle_rad)

    local xlt, ylt = rotateCompass(lb, tb, angle_rad)
    compassImage:bind()
    gl.Enable(gl.TEXTURE_2D)
    gl.Begin(gl.QUADS)
    gl.Color(1, 1, 1, 0.9 * alpha * opacity)
    gl.TexCoord(0, 1) gl.Vertex(xlb, ylb)
    gl.TexCoord(1, 1) gl.Vertex(xrb, yrb)
    gl.TexCoord(1, 0) gl.Vertex(xrt, yrt)
    gl.TexCoord(0, 0) gl.Vertex(xlt, ylt)
    gl.End()
    gl.Disable(gl.TEXTURE_2D)
end

local enableCompass = function(obj)
    local enable = false;
    if not empty(obj) then
        otype = obj:type()
        oname = obj:name()..' ';
         if otype == "spacecraft" then
            local ptype = obj:getinfo().parent:type()
            if ptype == "planet" or ptype == "moon" then
                enable = true;
            end
        elseif otype == "moon" then
            local ptype = obj:getinfo().parent:type()
            if ptype == "planet" then
                enable = true;
            end
        elseif otype == "planet" or oname == "Pluto " or otype == "location" then
            enable = true;
        end
    end
    return enable
end
----------------------------------------------
-- Set initial values
----------------------------------------------
width, height = celestia:getscreendimension()
compassFrameWidth, compassFrameHeight = boxWidth, 45

cCompass = ctext
compass_first_tick = true
ShiftLeft_first_tick = true

bgcolor = false
displonglat = true
switchable = false
measured_time = 560 
----------------------------------------------
-- Set initial values for compass orbit
---------------------------------------------- 
if not FramePerSec then
    FramePerSec = 30.00 -- default value
end

local obs = celestia:getobserver()
local obj = celestia:getselection()
local now = celestia:gettime()
local objpos = obj:getposition(now)
local obspos = obs:getposition()

local distance = obspos:distanceto(objpos)	-- radius orbit
local orbit_duration = 360 -- after test phase #1
local orbit_angle = math.rad(360.0) 
local orbit_steps = FramePerSec * orbit_duration -- Number of frames to process the graphics card in one round

local orbit_speed = 10 -- мой компьютер -- local orbit_speed = 2 -- What is it used for ?
local orbital_velocity = 2 * math.pi * distance / orbit_duration -- km/s
if math.floor(orbital_velocity) <1 then
	orbital_velocity = math.floor(orbital_velocity*1000)
    orbital_velocity = orbital_velocity/1000 -- 0.001 km/s = 1m/s
else
    orbital_velocity = math.floor(orbital_velocity) -- > 1km/s
end

local orbit_axis_D = celestia:newvector(1, 0, 0)
local orbit_axis_U = celestia:newvector(-1, 0, 0)
local orbit_axis_R = celestia:newvector(0, 1, 0)
local orbit_axis_L = celestia:newvector(0,-1, 0)

local orbit_duration_display = math.floor(orbit_duration)  --#off * AdjustAtPc) -- for adaptation at PC configuration, it's defined in config.lua, section Compass -- other method of adjustement
local orbit_rot_D = celestia:newrotation(orbit_axis_D, orbit_angle / orbit_steps)
local orbit_rot_U = celestia:newrotation(orbit_axis_U, orbit_angle / orbit_steps)
local orbit_rot_R = celestia:newrotation(orbit_axis_R, orbit_angle / orbit_steps)
local orbit_rot_L = celestia:newrotation(orbit_axis_L, orbit_angle / orbit_steps)

-- Get centerCompass initial value from 'config.lua'.
centerCompass = center_compass
-- Get compass size from 'config.lua'.
compass_size = math.min(150, math.max(50, compass_size))
compassW, compassH = compass_size, compass_size
retrograde_planets = {"Venus", "Uranus", "Pluto"}

---------------------------------------------------------
-- Set up and Draw the boxes
---------------------------------------------------------
compassBox = CXBox:new()
compassButton = CXBox:new() --  кнопка для вкл./выкл. компаса & джойстика
compassCheck = CXBox:new()

compassBox.Customdraw =
    function(this)
		compass_first_tick = false;
        local obs = celestia:getobserver()
        refObj = obs:getframe():getrefobject()
    end
--================================+
compassFrame = CXBox:new()
    :init(0, 0, 0, 0)
    :movable(false)
    :visible(false)
    :active(false)
    :attach(screenBox, width, height, 0, 0)

compassFrame.Customdraw =
    function(this)      

        local obs = celestia:getobserver()
        refObj = obs:getframe():getrefobject()
        if not(empty(refObj)) then
            if refObj:type() == "location" or refObj:type() == "spacecraft" then
                refObj = refObj:getinfo().parent;
            end
            az, elev = celutil.get_az_elev (obs, refObj)
            long, lat = celutil.get_long_lat(obs, refObj)
        end  

        longlatBox.Visible = true
        if not(empty(refObj)) then
            if celutil.get_dist(obs, refObj) < 1.02 * refObj:radius() then
                -----------------------------------------------------------------
                -- Planetarium mode.
                -----------------------------------------------------------------
                variantButton.Visible = true;               
                variantButton.Text = _("navigation mode")
                longlatBox.Active = true;
                compassFrameBox.Text = "^"
                if switchable == false then
                    displonglat = false;
                    switchable = true;
                end
                if alt_azimuthal_mode and math.abs(elev) < 88 then
                    ------------------------------------
                    -- Force Alt Azimuthal Mode.
                    -----------------------------------
                    celutil.force_up(obs, refObj)
                    if not(celestia:getaltazimuthmode()) then
                        celestia:setaltazimuthmode(true)
                        celestia:flash(_("Alt-azimuth mode enabled"))
                    end
                end
                -----------------------------------------------
                -- Avoid bug when centering reference object in planetarium mode.
                -----------------------------------------------
                if elev < -89.9 then
                    az = 0;
                end
                compass_angle = az;
                ---------------------------------------------
                -- When located at a (Geographic) Pole, make the
                -- compass constantly point to the opposite Pole.
                ---------------------------------------------
                if math.abs(lat) >= 89.995 then
                    compass_angle = math.abs(lat + 90)
                end
                --------------------------------------------
                -- Display the compass. We start displaying the compass on the second tick
                --  in order to hide a possible uggly resizing of the compass during the first tick.
                -------------------------------------------
                if not(compass_first_tick) then
                    compassFrameBox.Visible = true
                    drawCompass(compass_angle)
                end
            else
                --------------------------------------------
                -- Navigation mode.
                --------------------------------------------
                compassFrameBox.Text = "";
                compassFrameBox.Visible = false
                bgcolor = false;
                variantButton.Visible = true
                variantButton.Text = _("planetarium mode")
                longlatBox.Active = false
                displonglat = true
                switchable = false
            end
        end
        compassFrame:attach(screenBox, width-compassFrameWidth-5, compassBox.bb-compassFrameHeight, 5, height-compassBox.bb)
    end; 

--==================================
-- virtualPadBox.lua
--==================================

require "locale";
-------------------------------------------------
-- Load images.
-- Загрузить изображения.
-------------------------------------------------
padImageFilename = "../locale/"..lang.."/images/virtual_pad_"..lang..".png"
padImage = celestia:loadtexture(padImageFilename)
if not(padImage) then
    padImageFilename = "../images/virtual_pad.png"
    padImage = celestia:loadtexture(padImageFilename)
end

cursorImage = celestia:loadtexture("../images/pad_cursor.png")

----------------------------------------------
-- Define local function -- для дисплея Навигация
----------------------------------------------
local getObsMode = function(o)
    local obsCoordSys = o:getframe():getcoordinatesystem()
    local obj = o:getframe():getrefobject()
    return obsCoordSys, obj
end

local getTrackedObjName = function()
    if not (empty(trackedObj)) then
        trackedObjName = getlocalname(trackedObj)
    else
        trackedObjName = nil
    end
    return trackedObjName
end
local uly_to_km = 9460730.4725808

-- Get the rotation speed of the virtual pad from 'config.lua'.
-- Получить скорость виртуальной площадки из config.lua.
-- Define the rotation speed of virtual pad (can take values between 1 and 10).
-- Определить скорость вращения джойстика (может принимать значения от 1 до 10).

rotCoeff = measured_time / 1e4 / 360 --pad#51

-------------------------------------------------
-- Define some useful functions
-------------------------------------------------
setObsRotation = function(vectX, vectY)
    local obs = celestia:getobserver()
    local rotVect = celestia:newvector(vectY, vectX, 0)
    local rot = celestia:newrotation(rotVect, math.pi*rotCoeff)
    obs:rotate(rot)

    -- Set speed to the current value to make the observer move in the direction of the view.
    local speed = obs:getspeed()
    obs:setspeed(speed)
end

resetRotation = function()
    rotateUp = false
    rotateDown = false
    rotateLeft = false
    rotateRight = false
end
-------------------------------------------------
-- Set initial values
-------------------------------------------------
pad_first_tick = true

-- centerPad = center_virtual_pad

-------------------------------------------------------------------
-- Get the size of the virtual pad from 'config.lua'.
-- 
-- The size of the joystick (can take values from 50 to 120).
-- now: virtual_pad_size = 120

virtual_pad_size = math.min(120, math.max(50, virtual_pad_size))

-- the size of the pad of the joystick: 120х120 
padFrameW, padFrameH = virtual_pad_size, virtual_pad_size;
                     
-- the size of the touch zone for fixing the cursor 
arrowSize = padFrameW/5;

-- the size of the frame to move the cursor : 2*120/3 = 80 
padCursorFrameW, padCursorFrameH = 2*padFrameW/3+30, 2*padFrameH/3+30

-- the size of the touch zone to capture the cursor(the cursor is red dot): 
padCursorW, padCursorH = 15, 15

-- the moving distance of the cursor: (80 - 15 - 6)/2 ~= 30
cursorXMax, cursorYMax = (padCursorFrameW - padCursorW - 12 )/2, (padCursorFrameH - padCursorH - 12 )/2

----------------------------------------------
-- Set initial values
----------------------------------------------
local newwidth, newheight;
local panelFFrameWidth, panelFFrameHeight = 127, 173 -- Panel size panelFFrame/Размер панели panelFFrame
----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------
panelFFrame = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0,0.5,0.5,.8}) 
    --:fillcolor(csetbofill)   
    :textfont(titlefont)
    :textcolor({0.8, 0.8, 0.8, 0.9})
    :textpos("center")
    :movetext(0, -10)
    :movable(true)
    :clickable(false)    
    :visible(false)
    :attach(screenBox, (width-panelFFrameWidth)/2, height-panelFFrameHeight-2, (width-panelFFrameWidth)/2, 2)

panelFFrame.Customdraw = function(this)  

            if newwidth ~= width or newheight ~= height then
                    newwidth, newheight = width, height;                  
                    panelFFrame:attach(screenBox, (width-panelFFrameWidth)/2-1, 5-2, (width-panelFFrameWidth)/2, height-panelFFrameHeight-5+2) 
            end

--***************************************************************************
			if panelFFrame.Visible == true then
                -- Recalculates after the first start and the first display for be in phase with the selected object at startup
			    -- Refresh orbit parameters if distance as changed
 
                obs = celestia:getobserver()                -- observer=camera=you
                obj = celestia:getselection()               -- object selected
                now = celestia:gettime()                    -- actual time in celestia
                objpos = obj:getposition(now)              -- position of object
                obspos = obs:getposition()                  -- position of observer
                distance = obspos:distanceto(objpos)	-- radius orbit in km
                --orbit_duration as the start value or selection value
                orbital_velocity = 2 * math.pi * distance / orbit_duration -- km/s
				if math.floor(orbital_velocity) <1 then
					orbital_velocity = math.floor(orbital_velocity*10000)
					orbital_velocity = orbital_velocity/10000 -- 0.0001 km/s = 0.1m/s
				else
					orbital_velocity = math.floor(orbital_velocity) -- > 1km/s
				end
                -- display in TabloButton and Tablo2Button
                orbit_duration_display = math.floor(orbit_duration)
                TabloButton:text(_(orbit_duration_display))				
                Tablo2Button:text(_(orbital_velocity))
                -- refresh orbit
                orbit_steps = FramePerSec * orbit_duration
                orbit_rot_D = celestia:newrotation(orbit_axis_D, orbit_angle / orbit_steps)
                orbit_rot_U = celestia:newrotation(orbit_axis_U, orbit_angle / orbit_steps)
                orbit_rot_R = celestia:newrotation(orbit_axis_R, orbit_angle / orbit_steps)
                orbit_rot_L = celestia:newrotation(orbit_axis_L, orbit_angle / orbit_steps)
   			end
            
            textlayout:setfont(smallfont)
            textlayout:setfontcolor(cbutextoff) 
            textlayout:setpos(this.lb+46+1, this.tb -105-2) -- textlayout:setpos(this.lb+34, this.tb -110)
            textlayout:println(_("s/round")) -- textlayout:println(_(" 1 turn = sec.:"))
            
            textlayout:setfont(smallfont)
            textlayout:setfontcolor(cbutextoff) 
            textlayout:setpos(this.lb+49, this.tb-78) -- textlayout:setpos(this.lb+34, this.tb -79)
            textlayout:println(_("km/sec")) -- textlayout:println(_(" orbital km/s"))

end;
--================================
compassButton.Action = (function() 
        return
            function()
            compassFrame.Visible = not(compassFrame.Visible)
                if compassFrame.Visible then
                    compassCheck.Text = "o"  
                    panelFFrame.Visible = true
                    -- Disable information in the top right and bottom corners:
                    celestia:setoverlayelements{Time = false, Frame = false} 
                else
                    compassCheck.Text = ""
                    panelFFrame.Visible = false
                    compassFrameBox.Visible = false
                end              
            end
        end) ()

 closeCompassButton = CXBox:new()
        :init(0, 0, 0, 0) 
      --  :bordercolor(cbubordoff)
        :textfont(smallfont)
        :textcolor({1,0,0,0.5})
        :textpos("left")
        :movetext(0, 12)
        :text("x")
        :movable(false)
        :active(true)
        :attach(panelFFrame, 115, 160, 2, 2)

    closeCompassButton.Action = (function()
        return
            function()
                if compassFrame.Visible then
                    panelFFrame.Visible = false
                    compassFrame.Visible = false
                    panelAll24Check.Text = ""
                end             
         end
   end) ()

-- ==========================  the compass display  / дисплей компаса
longlatBox = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor({0.2,0.5,1,.5})
    :movable(false)
    :active(true) 
    :attach(panelFFrame, 3, 139, 3, 20)

longlatBox.Customdraw =
    function(this)
        -- Switch display between longitude-latitude and azimuth-elevation.
        textlayout:setfont(normalfont)
        textlayout:setlinespacing(normalfont:getheight())
        textlayout:setfontcolor(ctext)
        local obs = celestia:getobserver()

        if displonglat then
            for k,name in pairs(retrograde_planets) do
                if refObj:name().." " == name.." " then
                    long = -long;
                    lat = -lat;
                end
            end
			if long >= 0 then
				long_str = format:deg(math.abs(long))..''.._("E")
			else
				long_str = format:deg(math.abs(long))..''.._("W")
			end
			
			if lat >= 0 then
					lat_str = format:deg(math.abs(lat))..''.._("N")
			else
					lat_str = format:deg(math.abs(lat))..''.._("S")
			end

            textlayout:setfontcolor({1,1,1,0.3})
            textlayout:setpos(this.lb+3, this.tb-11)
           -- textlayout:println(_("Ln."))
            textlayout:setpos(this.lb+67, this.tb-11)
           -- textlayout:println(_("Lt."))
            textlayout:setfontcolor({0,1,0,0.9})
            textlayout:setpos(this.lb+5, this.tb-11)
            textlayout:println(long_str)         
            textlayout:setpos(this.lb+67, this.tb-11)
            textlayout:println(lat_str)
          --  textlayout:setfontcolor({0.2,0.8,1,1})  
          --  textlayout:setpos(this.lb+3, this.tb+7)

        else
            az_str = format:deg(az)            
            elev_str = format:deg(elev)  
            textlayout:setfontcolor({0.5,0.5,0.5,1})      
            textlayout:setpos(this.lb+3, this.tb-11)
            textlayout:println(_("Az."))                    -- Azimut/Азимут
            textlayout:setpos(this.lb+67, this.tb-11)
            textlayout:println(_("El."))                     -- Decline/Склонение
            textlayout:setfontcolor({0,1,0,0.9}) 
            textlayout:setpos(this.lb+20, this.tb-11)  
            textlayout:println(az_str)
            textlayout:setpos(this.lb+85, this.tb-11)  
            textlayout:println(elev_str)      
        end

		textlayout:setfontcolor({0.2,0.8,1,1})  
        textlayout:setpos(this.lb+3+20, this.tb+7)
        textlayout:println(_(" AUTOPILOT"))  -- textlayout:println(_("Compass & Joystick"))            
    end;

longlatBox.Action = (function()
    return
        function()
            displonglat = not(displonglat)
        end
    end) ()

-- =======================  button Landing/Start - кнопка Посадка/Старт
variantButton = CXBox:new()                           
    :init(0, 0, 0, 0) 
    :bordercolor({0, 1, 1, 0.3})
    :fillcolor({1,0,0,.5})
    --:fillcolor({0,.4,.6,.4})
    :textfont(normalfont)   
    :textcolor({1, 1 ,1, 0.7})   
    :textpos("center")
    :movetext(0, 7)
    :text(_("planetarium mode")) 
    :movable(false)
    :active(true) 
    :attach(panelFFrame, 3, 3, 3, 156)

variantButton.Action = (function()
    return
        function()
            local obs = celestia:getobserver()
              if variantButton.Text ==_("planetarium mode") then -- donc on est en mode navigation
                --if variantButton.Text ==_("pl.mode") then
                -- Switch to Planetarium Mode.
                obs:goto(refObj)
                obs:gotosurface(refObj)
            else
                -- Switch to Navigation Mode.
                celestia:select(refObj)
                obs:follow(refObj)
                local fov = obs:getfov()
                obs:gotodistance(refObj, 5.5 * refObj:radius() / (fov / 0.4688), 5)
            end
        end
    end) ()

-- ====================== frame compass / рамка для компаса
compassFrameBox = CXBox:new()
    :init(0, 0, 0, 0)   
    --:bordercolor({0, 1, 0, 1})   
    :textfont(normalfont)
    :textcolor({1, 1, 1, 0.2})
    :textpos("center")   
    :movetext(0, -6)
    :text("^")
    :visible(true)  
    :movable(false)
    :active(true)
    :attach(panelFFrame, 27, 41, 26, 57)
--=====================================
padCursorBox = CXBox:new()    
     -- :bordercolor({0,0.5,0.5,.5})   
      :fillimage(padImage)
      :init(0, 0, 0, 0)      
      :movable(false)  
      :attach(panelFFrame, 3+3, 3+17, 3+3-1, 53-17)

padCursorFrame = CXBox:new()                     -- pad to move the cursor / площадка для перемещения курсора
    :init(0, 0, 0, 0)    
    :visible(true)                                                   
    :movable(false)
    :clickable(false)   
    :active(false)
    :attach(padCursorBox,5,5,5,5)

padCursor = CXBox:new()
    :init(0, 0, 0, 0)
    :visible(true)                                                    
    :movable(true)
    :fillimage(cursorImage)
    :attach(padCursorFrame,    (padCursorFrameW-padCursorW)/2, (padCursorFrameH-padCursorH)/2,
                                          (padCursorFrameW-padCursorW)/2, (padCursorFrameH-padCursorH)/2)
padCursor.Customdraw = 
    function(this)
        if this.Move then
            ---------------------------------------------------------
            -- Stop rotations from arrow buttons.
            -- Остановить вращение для кнопок со стрелками.
            ---------------------------------------------------------

            resetRotation()

            ---------------------------------------------------------
            -- Define cursor coordinates and rotation vector.
            -- Определить координаты курсора и вращения вектора.
            ---------------------------------------------------------
             
            local cursorCenterX, cursorCenterY = padCursor:center()
            local frameCenterX, frameCenterY = padCursorFrame:center()
            local cursorX, cursorY = cursorCenterX - frameCenterX, cursorCenterY - frameCenterY;
            local rotVectX, rotVectY = cursorX / cursorXMax, cursorY / cursorYMax;

            --------------------------------------------------------
            -- Set rotation of observer.
            -- Установить вращение наблюдателя.
            --------------------------------------------------------

            if reverse_pad_pitch then
                setObsRotation(rotVectX, -rotVectY)
            else
                setObsRotation(rotVectX, rotVectY)
            end
        else
            --------------------------------------------------------
            -- Keep cursor centered in the Frame.
            -- Держать курсор в центре кадра.
            --------------------------------------------------------

            padCursor:attach(padCursorFrame,    (padCursorFrameW-padCursorW)/2, (padCursorFrameH-padCursorH)/2,
                                    (padCursorFrameW-padCursorW)/2, (padCursorFrameH-padCursorH)/2)
         end

        ------------------------------------------------------------
        -- Set rotation from arrow buttons.
        -- Установить вращения для кнопок со стрелками.
        ------------------------------------------------------------

        if rotateUp then
            padCursor:move(0, cursorYMax)
            if reverse_pad_pitch then
                setObsRotation(0, -1)
            else
                setObsRotation(0, 1)               
            end

        elseif rotateDown then
            padCursor:move(0, -cursorYMax)
            if reverse_pad_pitch then
                setObsRotation(0, 1)
            else
                setObsRotation(0, -1)
            end

        elseif rotateLeft then
            padCursor:move(-cursorXMax, 0)
            setObsRotation(-1, 0)

        elseif rotateRight then
            padCursor:move(cursorXMax, 0)
            setObsRotation(1, 0)
        end

    end;
----------------------------------
padUpButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0,0,1,1})
    :visible(true)
    :movable(false)
    :active(true) 
    :attach(padCursorBox, (padFrameW-arrowSize)/2+5, padFrameH-arrowSize-1, (padFrameW-arrowSize)/2+5, 12+1)

padUpButton.Action = (function()
        return
            function()
                if rotateUp then
                    resetRotation()
                else
                    resetRotation()
                    rotateUp = true;
                end
            end
        end) ()
-------------------------------------
padDownButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0,0,1,1})
    :visible(true)
    :movable(false)
    :active(true)
    :attach(padCursorBox, (padFrameW-arrowSize)/2+5, 12, (padFrameH-arrowSize)/2+5, padFrameW-arrowSize)

padDownButton.Action = (function()
        return
            function()
                if rotateDown then
                    resetRotation()
                else
                    resetRotation()
                    rotateDown = true;
                end
            end
        end) ()
-----------------------------------
padLeftButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0,0,1,1})
    :visible(true)
    :movable(false)
    :active(true)
    :attach(padCursorBox, 12, (padFrameH-arrowSize)/2+5, padFrameW-arrowSize, (padFrameH-arrowSize)/2+5)

padLeftButton.Action = (function()
        return
            function()
                if rotateLeft then
                    resetRotation()
                else
                    resetRotation()
                    rotateLeft = true;
                end
            end
        end) ()
-------------------------------------
padRightButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0,0,1,1})
    :visible(true)
    :movable(false)
    :active(true)
    :attach(padCursorBox, padFrameW-arrowSize-1, (padFrameH-arrowSize)/2+5, 12+1, (padFrameH-arrowSize)/2+5)

padRightButton.Action = (function()
        return
            function()
                if rotateRight then
                    resetRotation()
                else
                    resetRotation()
                    rotateRight = true;
                end
            end
        end) ()

-- ============================================
-- Compass : Flying around planets and other objects
-- ============================================
ShiftUpButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({1,0,0,1})
    :visible(true)
    :movable(false)
    :active(true) 
    :attach(padCursorBox, (padFrameW-arrowSize)/2+3, padFrameH-arrowSize+10, (padFrameW-arrowSize)/2+3, 3)

ShiftUpCheck = CXBox:new()
    :init(0, 0, 0, 0)
    :textfont(smallfont)
    :textcolor({0,0,1,1})
    :textpos("left")
    :movetext(0, 12)
    :text("")
    :movable(false)
    :active(true)  
    :attach(ShiftUpButton, 0, 0, 0, 0)

ShiftUpCheck.Action = (function()
    return
        function()
            display_ShiftUpButton = not display_ShiftUpButton;
                if display_ShiftUpButton then        
                   --ShiftDownCheck.Text = "";
                   --ShiftLeftCheck.Text = "";
                   --ShiftRightCheck.Text = "";
                   ShiftUpCheck.Text =  "o";
                else   
                   ShiftUpCheck.Text = "";
                end
        end
end) ()

ShiftUpButton.Customdraw = function()
        if display_ShiftUpButton then 
           obs:orbit(orbit_rot_U)
        end    
     end;
-- ===================
ShiftDownButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({1,0,0,1})
    :visible(true)
    :movable(false)
    :active(true)
    :attach(padCursorBox, (padFrameW-arrowSize)/2+3, 2, (padFrameH-arrowSize)/2+3, padFrameW-arrowSize+11)

ShiftDownCheck = CXBox:new()
    :init(0, 0, 0, 0)
    :textfont(smallfont)
    :textcolor({0,0,1,1})
    :textpos("left")
    :movetext(0, 12)
    :text("")
    :movable(false)
    :active(true)  
    :attach(ShiftDownButton, 0, 0, 0, 0)

ShiftDownCheck.Action = (function()
    return
        function()
            display_ShiftDownButton = not display_ShiftDownButton;
                if display_ShiftDownButton then           
                     --ShiftUpCheck.Text = "";
                     --ShiftLeftCheck.Text = "";
                     --ShiftRightCheck.Text = "";
                     ShiftDownCheck.Text =  "o";
                else    
                     ShiftDownCheck.Text = "";
                end
        end
end) ()

ShiftDownButton.Customdraw = function()
        if display_ShiftDownButton then 
           obs:orbit(orbit_rot_D)
        end 
     end;
-- ===================
ShiftRightButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({1,0,0,1})
    :visible(true)
    :movable(false)
    :active(true)
    :attach(padCursorBox, padFrameW-arrowSize+10, (padFrameH-arrowSize)/2+3, 2, (padFrameH-arrowSize)/2+3)

ShiftRightCheck = CXBox:new()
    :init(0, 0, 0, 0)
    :textfont(smallfont)
    :textcolor({0,0,1,1})
    :textpos("left")
    :movetext(-3, 8)
    :text("")
    :movable(false)
    :active(true)  
    :attach(ShiftRightButton, 0, 0, 0, 0)

ShiftRightCheck.Action = (function()
    return
        function()
            display_ShiftRightButton = not display_ShiftRightButton;
            if display_ShiftRightButton then         
                   ShiftRightCheck.Text =  "o";
            else   
                   ShiftRightCheck.Text = "";
            end
        end
end) ()

ShiftRightButton.Customdraw = function()
        if display_ShiftRightButton then
           obs:orbit(orbit_rot_R) 
        end     
end;
-- ==================
ShiftLeftButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({1,0,0,1})
    :visible(true)
    :movable(false)
    :active(true)
    :attach(padCursorBox, 2, (padFrameH-arrowSize)/2+3, padFrameW-arrowSize+11, (padFrameH-arrowSize)/2+3)

ShiftLeftCheck = CXBox:new()
    :init(0, 0, 0, 0)
    :textfont(smallfont)
    :textcolor({0,0,1,1})
    :textpos("left")
    :movetext(-3, 8)
    :text("")
    :movable(false)
    :active(true)  
    :attach(ShiftLeftButton, 0, 0, 0, 0)

ShiftLeftCheck.Action = (function()
    return
            function()
                display_ShiftLeftButton = not display_ShiftLeftButton;
                if display_ShiftLeftButton then 
                   --ShiftRightCheck.Text = "";
                   --ShiftUpCheck.Text = "";
                   --ShiftDownCheck.Text = "";
                    ShiftLeftCheck.Text =  "o";
                else   
                    ShiftLeftCheck.Text = "";
                end
            end
end) ()

ShiftLeftButton.Customdraw = function(this)
        if display_ShiftLeftButton then
           obs:orbit(orbit_rot_L) 
        end            
end;
----------------------------------------------------------
-- change the speed rotation
----------------------------------------------------------
SpeedRotBox = CXBox:new()                                                    
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
   -- :fillcolor({0.2,0.5,1,.5}) 
    :textfont(smallfont)   
    :textcolor({1,0,0,1})   
    :textpos("left")
    :movetext(-2, 12)
    :text(_(">>>")) 
    :movable(false)
    :active(true) 
    :attach(panelFFrame, 3, 126+2, 100-1, 37) -- panelFFrame width, height 127,173
   
SpeedRotBox.Action = (function() -- change speed of rotation by pre-selection
    return
        function()
            
 	       if orbit_duration > 360  then
                   orbit_duration = 360 -- 1 tour in 360 sec
                   rotCoeff = measured_time / 1e4 / 360 
           
	        elseif orbit_duration <= 360 and orbit_duration >300  then  -- Nearest pre-selection value
                   orbit_duration = 300 -- 1 tour in 300 sec
                   rotCoeff = measured_time / 1e4 / 300 

			elseif orbit_duration <= 300 and orbit_duration >270 then  -- Nearest pre-selection value
                   orbit_duration = 270 -- 1 tour in 270 sec
                   rotCoeff = measured_time / 1e4 / 270 

			elseif orbit_duration <= 270 and orbit_duration >240 then  -- Nearest pre-selection value
                   orbit_duration = 240 -- 1 tour in 240 sec
                   rotCoeff = measured_time / 1e4 / 240

			elseif orbit_duration <= 240 and orbit_duration >210 then  -- Nearest pre-selection value
                   orbit_duration = 210 -- 1 tour in 210 sec
                   rotCoeff = measured_time / 1e4 / 210 

			elseif orbit_duration <= 210 and orbit_duration >180 then  -- Nearest pre-selection value
                   orbit_duration = 180 -- 1 tour in 180 sec
                   rotCoeff = measured_time / 1e4 / 180 

			elseif orbit_duration <= 180 and orbit_duration >150 then  -- Nearest pre-selection value
                   orbit_duration = 150 -- 1 tour in 150 sec
                   rotCoeff = measured_time / 1e4 / 150

			elseif orbit_duration <= 150 and orbit_duration >120 then  -- Nearest pre-selection value
                   orbit_duration = 120 -- 1 tour in 120 sec
                   rotCoeff = measured_time / 1e4 / 120 

			elseif orbit_duration <= 120 and orbit_duration >90 then  -- Nearest pre-selection value
                   orbit_duration = 90 -- 1 tour in 90 sec
                   rotCoeff = measured_time / 1e4 / 90 

			elseif orbit_duration <= 90 and orbit_duration >60 then  -- Nearest pre-selection value
                   orbit_duration = 60 -- 1 tour in 60 sec
                   rotCoeff = measured_time / 1e4 / 60 

			elseif orbit_duration <= 60 and orbit_duration >40 then  -- Nearest pre-selection value
                   orbit_duration = 40 -- 1 tour in 40 sec
                   rotCoeff = measured_time / 1e4 / 20 

			elseif orbit_duration <= 40 and orbit_duration >20 then  -- Nearest pre-selection value
                   orbit_duration = 20  -- 1 tour in 20 sec
                   rotCoeff = measured_time / 1e4 / 20 

			elseif orbit_duration <= 20 and orbit_duration >10 then  -- Nearest pre-selection value
                   orbit_duration = 10 -- 1 tour in 10 sec
                   rotCoeff = measured_time / 1e4 / 10

			elseif orbit_duration <= 10 and orbit_duration >5 then  -- Nearest pre-selection value
                   orbit_duration = 5 -- 1 tour in 5 sec
                   rotCoeff = measured_time / 1e4 / 5 

			else
                  orbit_duration = 360 -- default
                  rotCoeff = measured_time / 1e4 / 360 
				  
			end
         end
    end) ()

MinusButton = CXBox:new()
    :init(0, 0, 0, 0)
   -- :fillcolor({1, 0 ,0, 1})
    :bordercolor(cbubordoff) 
    :textfont(smallfont)    
    :textcolor({1,0,0,1})   
    :textpos("left")
    :movetext(-2, 12)
    :text("<") 
    :visible(true)
    :movable(false)
    :active(true)
    :attach(panelFFrame, 100-2, 126+2, 17+1, 37)

MinusButton.Action = (function()
    return
        function()
			-- decrement of 10s or 5s
			if orbit_duration > 35 then
				orbit_duration = orbit_duration-10 -- edit --orbit_duration = orbit_duration-(10/AdjustAtPc) -- Decrement of 10
			elseif orbit_duration >= 10 then
				orbit_duration = orbit_duration-5 --# edit --orbit_duration = orbit_duration-(5/AdjustAtPc) -- Decrement of 5
			end
			 -- Low limit
            if orbit_duration <=5 then                --# edit --if orbit_duration <=5/AdjustAtPc then -- Low limit
                orbit_duration = 5 --# edit -- orbit_duration = 5/AdjustAtPc
            end
   		end
    end) ()	

PlusButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :textfont(smallfont)   
    :textcolor({1,0,0,1})   
    :textpos("left")
    :movetext(-2, 12)
    :text(">") 
    :visible(true)
    :movable(false)
    :active(true)
    :attach(panelFFrame, 114-1, 126+2, 3, 37)

PlusButton.Action = (function()
    return
        function()
			-- increment of 10s or 5s
			if orbit_duration >25 then
				orbit_duration = orbit_duration+10 --# edit -- orbit_duration = orbit_duration+(10/AdjustAtPc)
			elseif orbit_duration >= 0 then
				orbit_duration = orbit_duration+5 -- # edit -- orbit_duration = orbit_duration+(5/AdjustAtPc) -- increment of 5
            end
        end
end) ()

Minus2Button = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :textfont(smallfont)   
    :textcolor({1,0,0,1})   
    :textpos("left")
    :movetext(-4, 12)
    :text("<<") 
    :visible(true)
    :movable(false)
    :active(true)
	:attach(panelFFrame, 3, 117, 110, 48)
	
Minus2Button.Action = (function()
    return
        function()
			if orbit_duration >432 then
				orbit_duration = orbit_duration/1.2
            end
        end
end) ()	
	
Plus2Button = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :textfont(smallfont)   
    :textcolor({1,0,0,1})   
    :textpos("left")
    :movetext(-4, 12)
    :text(">>") 
    :visible(true)
    :movable(false)
    :active(true)
 	:attach(panelFFrame, 110, 117, 3, 48)  
	
Plus2Button.Action = (function()
    return
        function()
			if orbit_duration >=360 then
				orbit_duration = orbit_duration*1.2
            end
        end
end) ()	

TabloButton = CXBox:new()
    :init(0, 0, 0, 0)
    -- :fillcolor({0, 1 ,0, 1})
    -- :bordercolor(cbubordoff)
    :textfont(smallfont)  
    :textcolor({1, 0 ,0, 1})   
    :textpos("center") 
    :movetext(4, 10) -- :movetext(0, 10)
    :text(orbit_duration_display) -- display value
    :visible(true)
    :movable(false)
    :active(true)
    :attach(panelFFrame, 50, 52, 50, 110) --:attach(panelFFrame, 50, 47, 50, 115)

--===================================
--Determining the linear velocity of the flight over the planet
--===================================
Tablo2Button = CXBox:new()
	:init(0, 0, 0, 0)
	-- :bordercolor(cbubordoff)
	:textfont(smallfont)
	:textcolor({0,1,0,1})
	:textpos("center")
	:movetext(5, 10)
	:text(_(orbital_velocity))
	:visible(true)
	:movable(false)
	:active(true)
	:attach(panelFFrame, 42, 82, 42, 80)
--==========================================
-- set the parameter FramePerSec
--==========================================
local SetframepersecFrameWidth, SetframepersecFrameHeight = 300, 340
local newCoeff = 1.00
local newFramePerSec = FramePerSec
local newwidth1, newheight1;

FramePerSecButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(cFrameFill) 
    :textfont(smallfont)    
    :textcolor(color2) --{1,0,0,1} rouge
    :textpos("left")
    :movetext(0, 10)  
    :text(_("Set"))
    :movable(false)
    :active(true)  
    :attach(panelFFrame, 3, 37-17, 100, 126+17)  

FramePerSecButton.Action = (function()
    return
        function()
            SetframepersecFrame.Visible = not(SetframepersecFrame.Visible) 
             if not(SetframepersecFrame.Visible) then
                FramePerSecButton.Bordercolor = cbubordoff
                FramePerSecButton.Fillcolor = nil;       

             else
                FramePerSecButton.Bordercolor = cbubordon
                FramePerSecButton.Fillcolor = color0;
            end          
        end
end) ()

SetframepersecFrame = CXBox:new()
    :init(0, 0, 0, 0)
    :fillcolor(csetbofill)
    :bordercolor(cbubordoff)
    :textfont(titlefont)
    :textcolor({0.8, 0.8, 0.8, 0.9})
    :textpos("center")
    :text(_("Set parameter FramePerSec"))
    :movetext(0, -10)
    :movable(true)
    :visible(false)
    :attach(screenBox, (width-SetframepersecFrameWidth)/2+100, height-SetframepersecFrameHeight-2, (width-SetframepersecFrameWidth)/2-100, 2)

SetframepersecFrame.Customdraw = function(this)

        if  SetframepersecFrame_firstTickDone then
            if not(SetframepersecFrameCloseButtonDone) then
                SetframepersecFrameCloseButtonDone = true;
                makeCloseButton(SetframepersecFrameCloseButton, SetframepersecFrame,  SetframepersecFrameButton)
            end
        else
            SetframepersecFrame_firstTickDone = true;            
        end

        textlayout:setfont(normalfont)
        textlayout:setfontcolor({0,1,0,0.8})
        textlayout:setpos(this.lb+10, this.tb-50)
        textlayout:println(_("This coefficient depends on your graphics card"))
        textlayout:setpos(this.lb+10, this.tb-65)
        textlayout:println(_("We suggest you adjust its value."))
        textlayout:setpos(this.lb+10, this.tb-80)
        textlayout:println(_("The [+] buttons will increase the value, the [-] buttons"))            
        textlayout:setpos(this.lb+10, this.tb-95)
        textlayout:println(_("will decrease it."))
            
        textlayout:setpos(this.lb+10, this.tb-115)
        textlayout:println(_("Determine your preferred configuration in 'Set render'"))
        textlayout:setpos(this.lb+10, this.tb-130)
        textlayout:println(_("and always keep this config. for your tests."))
            
        textlayout:setfontcolor({1,0.6,0,1}) 
        textlayout:setpos(this.lb+10, this.tb-170)
        textlayout:println(_("Registered FramePerSec :"))             
        textlayout:setpos(this.lb+10, this.tb-190)
        textlayout:println(_("new FramePerSec :"))
            
        textlayout:setfontcolor({0.8, 0.8, 0.8, 0.9})
        textlayout:setpos(this.lb+10, this.tb-230)           
        textlayout:println(_("modify by 10%")) 

        textlayout:setpos(this.lb+105, this.tb-230)
        textlayout:println(_("modify by 1%")) 

        textlayout:setpos(this.lb+200, this.tb-230)
        textlayout:println(_("modify by 0.1%")) 
            
        RegisteredCoeffFrame.Text = math.floor (FramePerSec *1000)/1000
        NewCoeffFrame.Text = math.floor (newFramePerSec *1000)/1000
        textlayout:setfontcolor({0.8, 0.8, 0.8, 0.9})
        textlayout:setpos(this.lb+220, this.tb-180)
        textlayout:println(_(newCoeff))
 
        -- new dimensions of screen
        if newwidth1 ~= width or newheight1 ~= height then
            newwidth1, newheight1 = width, height;                  
            SetframepersecFrame:attach(screenBox, (width-SetframepersecFrameWidth)/2+100, height-SetframepersecFrameHeight-2, (width-SetframepersecFrameWidth)/2-100, 2) 
        end            
           
    end;

RegisteredCoeffFrame = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color01)
    :textfont(normalfont)   
    :textcolor(color0)
    :textpos("left")    
    :movetext(0, 5)
    :text(_(math.floor (FramePerSec*1000)/1000)) -- Round the value
    :movable(false)
    :active(true) 
    :attach(SetframepersecFrame, 160, SetframepersecFrameHeight-175, SetframepersecFrameWidth-210, SetframepersecFrameHeight-183) -- 160, 172, 105, 158

NewCoeffFrame = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color01)
    :textfont(normalfont)   
    :textcolor(color7)
    :textpos("left")    
    :movetext(0, 5)
    :text(_(math.floor (newFramePerSec*1000)/1000)) -- Round the value
    :movable(false)
    :active(true) 
    :attach(SetframepersecFrame, 160, SetframepersecFrameHeight-195, SetframepersecFrameWidth-210, SetframepersecFrameHeight-163)
 
Minus10Button = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color01)
    :textfont(titlefont)   
    :textcolor(color0)
    :textpos("center")    
    :movetext(0, 5)
    :text(_("-"))
    :movable(false)
    :active(true) 
    :attach(SetframepersecFrame, 37, SetframepersecFrameHeight-253, SetframepersecFrameWidth-52, SetframepersecFrameHeight-102)

Minus10Button.Action = (function()
    return 
           function()      
            newCoeff = newCoeff - 0.10
            newFramePerSec = math.floor (FramePerSec * (newCoeff)*1000)/1000 
        end
    end) ()
    
Plus10Button = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color01)
    :textfont(titlefont)   
    :textcolor(color0)
    :textpos("center")    
    :movetext(0, 3)
    :text(_("+"))
    :movable(false)
    :active(true) 
    :attach(SetframepersecFrame, 57, SetframepersecFrameHeight-253, SetframepersecFrameWidth-72,SetframepersecFrameHeight-102)
    
Plus10Button.Action = (function()
    return 
           function()      
            newCoeff = newCoeff + 0.10
            newFramePerSec = math.floor (FramePerSec * (newCoeff)*1000)/1000
        end
    end) ()
    
Minus1Button = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color01)
    :textfont(titlefont)   
    :textcolor(color0)
    :textpos("center")    
    :movetext(0, 5)
    :text(_("-"))
    :movable(false)
    :active(true) 
    :attach(SetframepersecFrame, 134, SetframepersecFrameHeight-253, SetframepersecFrameWidth-149, SetframepersecFrameHeight-102) 
  
Minus1Button.Action = (function()
    return 
           function()      
            newCoeff = newCoeff - 0.01
            newFramePerSec = math.floor (FramePerSec * (newCoeff)*1000)/1000 
        end
    end) ()
    
Plus1Button = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color01)
    :textfont(titlefont)   
    :textcolor(color0)
    :textpos("center")    
    :movetext(0, 3)
    :text(_("+"))
    :movable(false)
    :active(true) 
    :attach(SetframepersecFrame, 154, SetframepersecFrameHeight-253, SetframepersecFrameWidth-169, SetframepersecFrameHeight-102)
  
Plus1Button.Action = (function()
    return 
           function()      
            newCoeff = newCoeff + 0.01
            newFramePerSec = math.floor (FramePerSec * (newCoeff)*1000)/1000
        end
    end) ()

Minus01Button = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color01)
    :textfont(titlefont)   
    :textcolor(color0)
    :textpos("center")    
    :movetext(0, 5)
    :text(_("-"))
    :movable(false)
    :active(true) 
    :attach(SetframepersecFrame, 223, SetframepersecFrameHeight-253, SetframepersecFrameWidth-238, SetframepersecFrameHeight-102) 
  
Minus01Button.Action = (function()
    return 
           function()      
            newCoeff = newCoeff - 0.001
            newFramePerSec = math.floor (FramePerSec * (newCoeff)*1000)/1000 
        end
    end) ()
    
Plus01Button = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color01)
    :textfont(titlefont)   
    :textcolor(color0)
    :textpos("center")    
    :movetext(0, 3)
    :text(_("+"))
    :movable(false)
    :active(true) 
    :attach(SetframepersecFrame, 243, SetframepersecFrameHeight-253, SetframepersecFrameWidth-258, SetframepersecFrameHeight-102)
  
Plus01Button.Action = (function()
    return 
           function()      
            newCoeff = newCoeff + 0.001
            newFramePerSec = math.floor (FramePerSec * (newCoeff)*1000)/1000
        end
    end) ()
    
 ResetButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color01)
    :textfont(normalfont)   
    :textcolor(color7)
    :textpos("center")    
    :movetext(0, 3)
    :text(_("Reset"))
    :movable(false)
    :active(true)
    :attach(SetframepersecFrame, 20, SetframepersecFrameHeight-312, SetframepersecFrameWidth-60, SetframepersecFrameHeight-50) 

 ResetButton.Action = (function()
     return 
           function()      
            newCoeff = 1 -- reset without modification
            newFramePerSec = math.floor (FramePerSec *1000)/1000 -- reset without modification
        end
    end) ()
   
  SaveConfigButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color01)
    :textfont(normalfont)   
    :textcolor(color7)
    :textpos("center")    
    :movetext(0, 3)
    :text(_("Save in config"))
    :movable(false)
    :active(true)
    :attach(SetframepersecFrame, 85, SetframepersecFrameHeight-312, SetframepersecFrameWidth-175, SetframepersecFrameHeight-50)  

  SaveSessionButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color01)
    :textfont(normalfont)   
    :textcolor(color7)
    :textpos("center")    
    :movetext(0, 3)
    :text(_("Save in session"))
    :movable(false)
    :active(true)
    :attach(SetframepersecFrame, 190, SetframepersecFrameHeight-312, SetframepersecFrameWidth-285, SetframepersecFrameHeight-50)  
    
SaveSessionButton.Action = (function()
    return 
           function()      
                FramePerSec = math.floor (newFramePerSec *1000)/1000 -- validate new FramePerSec in this session of celestia
                newCoeff = 1 -- reset
                newFramePerSec = math.floor (FramePerSec *1000)/1000 -- reset
            end
    end) ()  

SaveConfigButton.Action = (function()
     return 
           function() 
                -- save FramePerSec in config
                -- set the parameters
                FramePerSec = math.floor (newFramePerSec *1000)/1000 -- validate new FramePerSec in this session of celestia, round 3 decimals
                newCoeff = 1 -- reset
                newFramePerSec = FramePerSec
                local StrFramePerSec = "--"  -- for fixe to string variable              
                StrFramePerSec =  FramePerSec
                local lg = string.len(StrFramePerSec)
                -- formatage de StrFramePerSec --------------------
                if lg < 2 then
                    StrFramePerSec = "0"..StrFramePerSec..".000"
                elseif lg == 2 then
                    StrFramePerSec = StrFramePerSec..".000"
                elseif lg == 3 then
                    StrFramePerSec = StrFramePerSec.."000"
                elseif lg == 4 then
                    StrFramePerSec = StrFramePerSec.."00"
                elseif lg == 5 then
                    StrFramePerSec = StrFramePerSec.."0"
                elseif lg == 6 then
                    StrFramePerSec = StrFramePerSec
                end
                -----------------------------------------------------------
                local entryFramepersec = "FramePerSec = "..StrFramePerSec.." "    -- the minimal text to save 
                -- File in the software root or  '/' is the only meaning allowed for the slash for change directory
                local fichiertxt = "lua-applications/lua_universal_tools_v5/config.txt" -- after test changes config.txt by config.lua
                local entryFind = "" -- init
                local line2 = ""
                 
                ----------------------------------------------------------------------------------------------------
                --  first tick : finds the complet line of entry 
                for line in io.lines (fichiertxt) do
                    local posfind = string.find(line, "FramePerSec = ", 1)   -- finds entry at position 1 of line, lenght 14
                    if posfind == 1 then
                        entryFind = line                -- saves the complet finded entry
                    end
                end -- closes file automatically
                
                if entryFind =="" then -- Missing entry 
                    local fichiertxtForAdd = io.open(fichiertxt, "a") -- opens file in write mode add
                    entryFind = entryFramepersec.."-- edit "..os.date("%d/%m/%Y %H:%M:%S") -- creates entry at the file's end
                    io.output(fichiertxtForAdd):write(entryFind) -- Written in the output file
                    io.close(fichiertxtForAdd) -- Closes the file
                end                
                ----------------------------------------------------------------------------------------------------
                -- second tick : searches and changes string corresponding at entryFind
                
                io.input(fichiertxt)
                             -- reads the file in totality
                line2 = io.read("*a")
                            -- position in file
                local posfind, posend = string.find(line2, entryFind, 1)            -- 20 characters "FramePerSec = 30 -- " ou "FramePerSec = 30.000"
                            -- "FramePerSec = xx.xxx" or "FramePerSec = xx.xx "
                local entryFind2 = string.sub (entryFind, 1, 20)
                if string.sub (entryFind, 17, 20) == " -- " then          -- add 05-04-2017
                    entryFind3 = string.sub (entryFind, 1, 16)  -- add 05-04-2017
                else
                    entryFind3 = entryFind2
                end                                                                     -- add 05-04-2017
                            -- Processing '--' characters not recognized in entryFind in the 'write' statement
                local entryFind4 = string.sub (entryFind, 21, string.len(entryFind))    -- from character 21 to line's end, '--' is ignored                              
                            -- changes the text entryFind2 by entryFramepersec in 'line2' (total file)
                line2  = string.gsub(line2, entryFind3, entryFramepersec ) -- modified 05-04-2017 entryFind2 by entryFind3
                            -- changes the text entryFind4 by comment in 'line2' (total file)
                if entryFind4 == "" then 
                    entryFind4 = entryFind3  -- modified 05-04-2017 entryFind2 by entryFind3
                    line2  = string.gsub(line2, entryFramepersec, entryFramepersec.."-- edit "..os.date("%d/%m/%Y %H:%M:%S")) -- add a comment
                else
                    line2  = string.gsub(line2, entryFind4, "-- edit "..os.date("%d/%m/%Y %H:%M:%S")) -- modifies the comment finded
                end 
                io.output(fichiertxt)
                io.write(line2) -- updates the file in totality
                io.close() -- Closes the file
                --------------------------------------------------------------------------------------------------------------

                celestia:print(_("New setting saved!"))                
        end
    end) () 
