-- ===================================
-- solarSystemBox.lua
-- Author - Vincent Giangiulio a.k.a Vincent
-- Edit by Lygin Gennady a.k.a Croc 05.03.2015
-- Integration by Lygin Gennady a.k.a Croc 18.03.2017
-- Edit by Patrick Ziegler a.k.a. Gironde 07-04-2017
-- ===================================
require "locale";
require "closeButton";
require "scrollButton";
----------------------------------------------
-- Load images.
----------------------------------------------
startImage = celestia:loadtexture("../images/start_button.png");
endImage = celestia:loadtexture("../images/end_button.png");
----------------------------------------------
-- Define local functions
----------------------------------------------
local makeChildrenBox = function(childrenBox, k, leftOffset)                               -- окна для названий объектов
    childrenBox[k] = CXBox:new()
        :init(0, 0, 0, 0)
        :textfont(normalfont)
        :textcolor(cbutextoff)
        :movetext(0, 6)
        :text("")
        :movable(false)
        :active(true)
        :attach(solsysFrame, leftOffset+5, solsysFrameHeight-52-(18*k), solsysFrameWidth-leftOffset-85, 37+(18*k))
end

local makeEtcBox = function(leftOffset)                                                         -- окна в конце столбцов с названиями
    etcBox = CXBox:new()
        :init(0, 0, 0, 0)
        :textfont(normalfont)
        :textcolor(cclotext)
        :movetext(1, 12)
        :text("...")
        :movable(false)
        :attach(solsysFrame, leftOffset+5, 20, solsysFrameWidth-leftOffset-85, solsysFrameHeight-30)
    return etcBox;
end

local makePrefixBox = function(prefixBox, k, leftOffset)                                    -- окна с плюсиками
    prefixBox[k] = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(cbubordoff)
        :textfont(smallfont)
        :textcolor(cbutextoff)
       -- :textpos("center")      -- edit
        :movetext(-3, 10) 
        :text("+")
        :movable(false)
        :active(true)
        :visible(false)     
        :attach(solsysFrame, leftOffset-3, solsysFrameHeight-50-(18*k), solsysFrameWidth-leftOffset-7, 40+(18*k))
end

local makeStartBox = function(startBox, k, leftOffset)                                           -- левые кнопки начала старта
    startBox[k] = CXBox:new()
        :init(0, 0, 0, 0)
        :fillimage(startImage)
        :movable(false)
        :active(true)
        :visible(false)       
        :attach(solsysFrame, leftOffset+196, solsysFrameHeight-50-(18*k), solsysFrameWidth-leftOffset-204, 41+(18*k))
end

local makeEndBox = function(endBox, k, leftOffset)                                             -- правые кнопки конец работы косм. аппарата
    endBox[k] = CXBox:new()
        :init(0, 0, 0, 0)
        :fillimage(endImage)
        :movable(false)
        :active(true)
        :visible(false)      
        :attach(solsysFrame, leftOffset+186+20, solsysFrameHeight-50-(18*k), solsysFrameWidth-leftOffset-194-20, 41+(18*k)) 
end

local makeChildScrollbarFrame = function(leftOffset)                                       -- рамки для слайдеров
    childScrollbarFrame = CXBox:new()
        :visible(true)       
        :attach(solsysFrame, leftOffset-16, 30, solsysFrameWidth-leftOffset+16-childScrollbarFrameW, solsysFrameHeight-30-childScrollbarFrameH)

    return childScrollbarFrame;
end

local makeChildScrollbarBox = function(childScrollbarFrame, leftOffset, nChildren)           -- рамки для ползунков
    childScrollbarBox = CXBox:new()
        :bordercolor(checkBcolor)
        :fillcolor(cscrollfill)
        :visible(true)
        :movable(true)
        :attach(childScrollbarFrame, 0, (nChildren-nChildrenMax)*solsysScrollCoeff, 0, 3)

    return childScrollbarBox;
end

local makeChildScrollUpButton = function(childScrollbarFrame)                       -- тупик для ползунка вверху
    childScrollUpButton = CXBox:new()
        :init(0, 0, 0, 0)
        :textfont(titlefont)
        :textcolor(cclotext)
        --:textpos("center")
        :movetext(-4, -1)
        :text(_("ˆ"))
        :movable(false)
        :attach(childScrollbarFrame, 0, childScrollbarFrameH, 0, -11);
    return childScrollUpButton;
end

local makeChildScrollDownButton = function(childScrollbarFrame)                    -- тупик для ползунка внизу
    childScrollDownButton = CXBox:new()
        :init(0, 0, 0, 0)
        :textfont(titlefont)
        :textcolor(cclotext)
        --:textpos("center")
        :movetext(-4, -1)
        :text(_("ˇ"))
        :movable(false)
        :attach(childScrollbarFrame, 0, -11, 0,childScrollbarFrameH);    -- Inverted but OK (not active).
    return childScrollDownButton;
end

local refreshChildScrollbarFrame = function(childScrollbarFrame, childScrollbarBox)
    local lb3,bb3,rb3,tb3 = childScrollbarFrame:bounds();
    local lb4,bb4,rb4,tb4 = childScrollbarBox:bounds();
    childScroll = math_round((tb3-tb4-3)/solsysScrollCoeff);
    return childScroll
end

local refreshChildScrollbarBox = function(childScrollbarFrame, childScrollbarBox, childScroll, childScrollbarBoxH, nChildren)
    if resetScrollPos then
        if nChildren < 4000 then
            solsysScrollCoeff = 25*math.log(nChildren)/(nChildren-nChildrenMax)
        else
            solsysScrollCoeff = 200/(nChildren-nChildrenMax)
        end
        childScrollbarBoxH = childScrollbarFrameH-6-(nChildren-nChildrenMax)*solsysScrollCoeff
        childScrollbarBox:attach(childScrollbarFrame, 0, childScrollbarFrameH-childScrollbarBoxH-3, 0, 3)
    end
end

local setScrollbarVisibility = function(childScrollbarFrame, nChildren)
    if nChildren > nChildrenMax then
        childScrollbarFrame.Visible = true;
    else
        childScrollbarFrame.Visible = false;
    end
end

local setEtcBoxVisibility = function(childEtcBox, childScroll, nChildren)
    if childScroll < nChildren-nChildrenMax then
        childEtcBox.Visible = true;
    else
        childEtcBox.Visible = false;
    end
end

local getSolarSystemElements = function()
    local sel = celestia:getselection();
    if selection then
        parent = sel;
        while parent:type() ~= "star" do
            parent = parent:getinfo().parent
        end
        children = parent:getchildren();
        for k, v in ipairs(children) do
            if v:type() == "invisible" or v:name() == "" then
                table.remove(children, k)
            end
        end
        nchildren = table.getn(children);
        return parent, children, nchildren;
    end
end

local deleteChildrenNames = function()
    for k = 1, nChildrenMax do
        planetBox[k].Text = "";
        dwarfplanetBox[k].Text = "";
        moonBox[k].Text = "";
        minormoonBox[k].Text = "";
        spacecraftBox[k].Text = "";
    end
end

local removeNumberFromName = function(name)
    pos = string.find(name, ' ');
    if pos then
        num =  tonumber(string.sub(name, 0, pos-1));
        if num and (num < 1998 or num > 2010) then
            name = string.sub(name, pos + 1);
        end
    end
    return name;
end

local resetScroll = function()
    dwarfplanetScroll = 0;
    moonScroll = 0;
    minormoonScroll = 0;
    spacecraftScroll = 0;
end

local resetAllScrolls = function()
    planetScroll = 0;
    dwarfplanetScroll = 0;
    moonScroll = 0;
    minormoonScroll = 0;
    spacecraftScroll = 0;
end

local emptyClasses = function()
    dwarfplanets = {};
    moons = {};
    minormoons = {};
    spacecraft = {};
end

local emptyAllClasses = function()
    planets = {};
    dwarfplanets = {};
    moons = {};
    minormoons = {};
    spacecraft = {};
end

local setPrefix = function(prefixBox, k, nchild)
    if nchild > 0 then
        prefixBox[k].Visible = true;
    else
        prefixBox[k].Visible = false;
    end
end

local setStart = function(startBox, k, startdate)
    if startdate and not string.find(startdate, "INF") then
        startBox[k].Visible = true;
    else
        startBox[k].Visible = false;
    end
end

local setEnd = function(endBox, k, enddate)
    if enddate and not string.find(enddate, "INF") then
        endBox[k].Visible = true;
    else
        endBox[k].Visible = false;
    end
end

local setBoxTextColorOff = function(box)
    for k, v in pairs(box) do
        v.Textcolor = cbutextoff;
    end
end

local setBoxBorderColorOff = function(box)
    for k, v in pairs(box) do
        v.Bordercolor = cbubordoff;
    end
end

local setObjectColorsOff = function()
    starBox.Textcolor = cbutextoff;
    setBoxTextColorOff(planetBox);
    setBoxTextColorOff(dwarfplanetBox);
    setBoxTextColorOff(moonBox);
    setBoxTextColorOff(minormoonBox);
    setBoxTextColorOff(spacecraftBox);
end

local setPrefixColorsOff = function()
    setBoxTextColorOff(planetPrefixBox);
    setBoxTextColorOff(dwarfplanetPrefixBox);
    setBoxTextColorOff(moonPrefixBox);
    setBoxTextColorOff(minormoonPrefixBox);
    setBoxTextColorOff(spacecraftPrefixBox);

    setBoxBorderColorOff(planetPrefixBox);
    setBoxBorderColorOff(dwarfplanetPrefixBox);
    setBoxBorderColorOff(moonPrefixBox);
    setBoxBorderColorOff(minormoonPrefixBox);
    setBoxBorderColorOff(spacecraftPrefixBox);
end

----------------------------------------------
-- Set initial values
----------------------------------------------
local star, starChildren, nStarChildren = getSolarSystemElements();
solsysFrameWidth, solsysFrameHeight = 735, 260; 

childScrollbarFrameW, childScrollbarFrameH = 9, 175;

  starLeftOffset = 7                              -- звёзды
  planetLeftOffset = 39                         -- планеты
  dwarfplanetLeftOffset = 149                -- малые планеты
  moonLeftOffset = 259;                        -- луны
  minormoonLeftOffset = 396;                -- малые луны
  spacecraftLeftOffset = 506;                 -- космические корабли

  nChildrenMax = 10;                                 -- максимальное количество строк в каталоге
  
resetAllScrolls();
emptyAllClasses();

planetBox = {};
dwarfplanetBox = {};
moonBox = {};
minormoonBox = {};
spacecraftBox = {};

planetPrefixBox = {};
dwarfplanetPrefixBox = {};
moonPrefixBox = {};
minormoonPrefixBox = {};
spacecraftPrefixBox = {};

spacecraftStartBox = {};
spacecraftEndBox = {};

nPlanets = 0;
nDwarfPlanets = 0;
nMoons = 0;
nMinorMoons = 0;
nSpacecraft = 0;

solsysScrollCoeff = 10;

planetScrollbarBoxH = 0;
dwarfplanetScrollbarBoxH = 0;
moonScrollbarBoxH = 0;
minormoonScrollbarBoxH = 0;
spacecraftScrollbarBoxH = 0;
local newwidth, newheight

----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------

solarSystemBox = CXBox:new()                                         
    :init(0, 0, 0, 0)
    --:fillcolor(cFrameFill)
    :movable(false)
    :attach(screenBox, width, height, 0, 0);

--[[
solsysButton = CXBox:new()                 -- клавиша на 2-й строке команд в верхнем правом углу экрана
    :init(0, 0, 0, 0)
    :bordercolor({0.2,0.8,1,.6}) 
    :fillcolor(cFrameFill)
    :textfont(smallfont)  
    :textcolor({0.2,0.8,1,1})
    :textpos("left")
    :movetext(-3, 10)  
    :text(_("Browser"))
    :movable(false)
    :active(true) 
   :attach(menuLine3Box, -44, 0, 143, 0)

solsysButton.Action = (function()
    return
        function()
            starName = nil;
            solsysFrame.Visible = not(solsysFrame.Visible)
            if solsysFrame.Visible then
                solsysFrame:orderfront()
                solsysButton.Bordercolor = {1,0,0,.6}
            else
                solsysButton.Bordercolor = {0.2,0.8,1,.6}
            end
        end
    end) ()
]]
------ frame ---------------------------------------------------------------------------------------------------
solsysFrame = CXBox:new()                                         -- всплывающая панель- каталог
    :init(0, 0, 0, 0)
    :fillcolor(csetbofill)
    --:bordercolor(cbubordoff)
  --  :movable(false)
    :movable(true)
    :visible(false)
    :attach(screenBox, (width-solsysFrameWidth)/2, 0, (width-solsysFrameWidth)/2, height-solsysFrameHeight);
solsysFrame.Customdraw =
    function(this)
        local sel = celestia:getselection();
        if selection and sel:type() ~= "galaxy" and sel:type() ~= "nebula" and sel:type() ~= "opencluster" then
            star, starChildren, nStarChildren = getSolarSystemElements();
            resetScrollPos = false;
            starBox.Visible = true;
            setObjectColorsOff();
            setPrefixColorsOff();
            if parentObj and parentObjChanged then
                parentObjChanged = false;
                resetScrollPos = true;
                emptyClasses();
                resetScroll();
                children = parentObj:getchildren();
                for k, v in ipairs(children) do
                    if v:name() ~= "" and v:visible() then

                        if v:type() == "moon" then
                            table.insert(moons, v);

                        elseif v:type() == "minormoon" then
                            table.insert(minormoons, v);
                        elseif v:type() == "spacecraft" then
                            table.insert(spacecraft, v);
                        end
                    end
                end
                nDwarfPlanets = 0;
                nMoons = table.getn(moons);
                nMinorMoons = table.getn(minormoons);
                nSpacecraft = table.getn(spacecraft);
                parentObj_str = _("Bodies orbiting")..' '..limitStrWidth(getlocalname(parentObj), normalfont, 100)..' :';
            end
            if star and starName ~= getlocalname(star) then
                starName = getlocalname(star);
                resetScrollPos = true;
                starBox.Text = limitStrWidth(starName, normalfont, 100);
                parentObj = star;
                emptyAllClasses();
                resetAllScrolls();
                for k, v in ipairs(starChildren) do
                    if v:name() ~= "" and v:visible() then
                        if v:type() == "planet" then
                            table.insert(planets, v)
                        elseif v:type() == "dwarfplanet" then
                            table.insert(dwarfplanets, v)
                        elseif v:type() == "spacecraft" then
                            table.insert(spacecraft, v)
                        end
                    end
                end
                nPlanets = table.getn(planets);
                nDwarfPlanets = table.getn(dwarfplanets);
                nMoons = 0;
                nMinorMoons = 0;
                nSpacecraft = table.getn(spacecraft);
                if nSpacecraft == 0 then
                    parentObj_str = "";
                else
                    parentObj_str = _("Other bodies orbiting")..' '..limitStrWidth(getlocalname(parentObj), normalfont, 100)..' :';
                end
            end
            -- We start displaying the text on the second tick in order to
            -- hide a possible uggly resizing of the box during the first tick.
            if solsys_firstTickDone then
                textlayout:setfont(normalfont);
                textlayout:setfontcolor({0,1,1,0.5}); 
                textlayout:setpos(this.lb+moonLeftOffset+10+100, this.tb-15-5);
                textlayout:println(parentObj_str);
                textlayout:setfontcolor({0,1,1,0.5}); 

                if not(solsysCloseButtonDone) then
                    solsysCloseButtonDone = true;
                    makeCloseButton(solsysCloseButton, solsysFrame, solsysButton);
                end

                if nPlanets > 0 then
                    textlayout:setpos(this.lb+starLeftOffset+5, this.tb-20);                             -- заголовок "Выбранная звезда"           
                    textlayout:println(_("Selected star:"));

                    textlayout:setpos(this.lb+planetLeftOffset-15, this.tb-40);                           -- заголовок "Планеты"  
                    textlayout:println(_("Planets").." :");
                end

                if nDwarfPlanets > 0 then
                    textlayout:setpos(this.lb+dwarfplanetLeftOffset-35, this.tb-40);                    -- заголовок "Малые планеты"  
                    textlayout:println(_("Dwarf Planets").." :");
                end

                if nMoons > 0 then
                    textlayout:setpos(this.lb+moonLeftOffset-35, this.tb-40);                             -- заголовок "Луны"  
                    textlayout:println(_("Moons").." :");
                end

                if nMinorMoons > 0 then
                    textlayout:setpos(this.lb+minormoonLeftOffset-35, this.tb-40);                     -- заголовок "Малые луны"   
                    textlayout:println(_("Minor Moons").." :");
                end

                if nSpacecraft > 0 then
                    textlayout:setpos(this.lb+spacecraftLeftOffset-15, this.tb-40);                      -- заголовок "Косм. корабли"  
                    textlayout:println(_("Spacecraft").." :");
                end
            else
                solsys_firstTickDone = true;
            end
            for k = 1, nChildrenMax do
                if planets[k+planetScroll] then
                    planetBox[k].Visible = true;
                    local child = planets[k+planetScroll]:getchildren();
                    local nchild = table.getn(child);
                    setPrefix(planetPrefixBox, k, nchild);
                    planetBox[k].Text = limitStrWidth(getlocalname(planets[k + planetScroll]), normalfont, 77);
                else
                    planetBox[k].Visible = false;
                    planetPrefixBox[k].Visible = false;
                end
                if dwarfplanets[k+dwarfplanetScroll] then
                    dwarfplanetBox[k].Visible = true;
                    local child = dwarfplanets[k+dwarfplanetScroll]:getchildren();
                    local nchild = table.getn(child);
                    setPrefix(dwarfplanetPrefixBox, k, nchild);
                    dwarfplanetBox[k].Text = limitStrWidth(removeNumberFromName(getlocalname(dwarfplanets[k + dwarfplanetScroll])), normalfont, 77);
                else
                    dwarfplanetBox[k].Visible = false;
                    dwarfplanetPrefixBox[k].Visible = false;
                end

                if moons[k+moonScroll] then
                    moonBox[k].Visible = true;
                    local child = moons[k+moonScroll]:getchildren();
                    local nchild = table.getn(child);
                    setPrefix(moonPrefixBox, k, nchild);
                    moonBox[k].Text = limitStrWidth(getlocalname(moons[k+moonScroll]), normalfont, 200);
                else
                    moonBox[k].Visible = false;
                    moonPrefixBox[k].Visible = false;
                end
                if minormoons[k+minormoonScroll] then
                    minormoonBox[k].Visible = true;
                    local child = minormoons[k+minormoonScroll]:getchildren();
                    local nchild = table.getn(child);
                    setPrefix(minormoonPrefixBox, k, nchild);
                    minormoonBox[k].Text = limitStrWidth(getlocalname(minormoons[k+minormoonScroll]), normalfont, 200);
                else
                    minormoonBox[k].Visible = false;
                    minormoonPrefixBox[k].Visible = false;
                end

                if spacecraft[k+spacecraftScroll] then
                    spacecraftBox[k].Visible = true;
                    local child = spacecraft[k+spacecraftScroll]:getchildren();
                    local nchild = table.getn(child);
                    setPrefix(spacecraftPrefixBox, k, nchild);
                    spacecraftBox[k].Text = limitStrWidth(getlocalname(spacecraft[k+spacecraftScroll]), normalfont, 200);
                    local startdate = spacecraft[k+spacecraftScroll]:getinfo().lifespanStart;
                    local enddate = spacecraft[k+spacecraftScroll]:getinfo().lifespanEnd;
                    setStart(spacecraftStartBox, k, startdate);
                    setEnd(spacecraftEndBox, k, enddate);
                else
                    spacecraftBox[k].Visible = false;
                    spacecraftPrefixBox[k].Visible = false;
                    spacecraftStartBox[k].Visible = false;
                    spacecraftEndBox[k].Visible = false;
                end
            end

            setScrollbarVisibility(planetScrollbarFrame, nPlanets);
            setScrollbarVisibility(dwarfplanetScrollbarFrame, nDwarfPlanets);
            setScrollbarVisibility(moonScrollbarFrame, nMoons);
            setScrollbarVisibility(minormoonScrollbarFrame, nMinorMoons);
            setScrollbarVisibility( spacecraftScrollbarFrame, nSpacecraft);

            setEtcBoxVisibility(planetEtcBox, planetScroll, nPlanets);
            setEtcBoxVisibility(dwarfplanetEtcBox, dwarfplanetScroll, nDwarfPlanets);
            setEtcBoxVisibility(moonEtcBox, moonScroll, nMoons);
            setEtcBoxVisibility(minormoonEtcBox, minormoonScroll, nMinorMoons);
            setEtcBoxVisibility(spacecraftEtcBox, spacecraftScroll, nSpacecraft);
        else
            star = nil;
            starName = nil;
            parentObj = nil;
            deleteChildrenNames();
            setPrefixColorsOff();
            starBox.Visible = false;
            planetScrollbarFrame.Visible = false;
            planetEtcBox.Visible = false;
            dwarfplanetScrollbarFrame.Visible = false;
            dwarfplanetEtcBox.Visible = false;
            moonScrollbarFrame.Visible = false;
            moonEtcBox.Visible = false;
            minormoonScrollbarFrame.Visible = false;
            minormoonEtcBox.Visible = false;
            spacecraftScrollbarFrame.Visible = false;
            spacecraftEtcBox.Visible = false;
            for k = 1, nChildrenMax do
                planetPrefixBox[k].Visible = false;
                dwarfplanetPrefixBox[k].Visible = false;
                moonPrefixBox[k].Visible = false;
                minormoonPrefixBox[k].Visible = false;
                spacecraftPrefixBox[k].Visible = false;
                spacecraftStartBox[k].Visible = false;
                spacecraftEndBox[k].Visible = false;
            end
        end
        if newselection then
            setObjectColorsOff();
            setPrefixColorsOff();
        end
        if starName == getlocalname(sel) then
            starBox.Textcolor = cbutexton;
        end
        for k = 1, nChildrenMax do
            if getlocalname(planets[k+planetScroll]) == getlocalname(sel) then
                planetBox[k].Textcolor = cbutexton;
            end
            if getlocalname(dwarfplanets[k+dwarfplanetScroll]) == getlocalname(sel) then
                dwarfplanetBox[k].Textcolor = cbutexton;
            end

            if getlocalname(moons[k+moonScroll]) == getlocalname(sel) then
                moonBox[k].Textcolor = cbutexton;
            end
            if getlocalname(minormoons[k+minormoonScroll]) == getlocalname(sel) then
                minormoonBox[k].Textcolor = cbutexton;
            end

            if getlocalname(spacecraft[k+spacecraftScroll]) == getlocalname(sel) then
                spacecraftBox[k].Textcolor = cbutexton;
            end
            if getlocalname(planets[k+planetScroll]) == getlocalname(parentObj) then
                planetPrefixBox[k].Textcolor = cbutexton;
                planetPrefixBox[k].Bordercolor = cbubordon;
            end
            if getlocalname(dwarfplanets[k+dwarfplanetScroll]) == getlocalname(parentObj) then
                dwarfplanetPrefixBox[k].Textcolor = cbutexton;
                dwarfplanetPrefixBox[k].Bordercolor = cbubordon;
            end

            if getlocalname(moons[k+moonScroll]) == getlocalname(parentObj) then
                moonPrefixBox[k].Textcolor = cbutexton;
                moonPrefixBox[k].Bordercolor = cbubordon;
            end
            if getlocalname(minormoons[k+minormoonScroll]) == getlocalname(parentObj) then
                minormoonPrefixBox[k].Textcolor = cbutexton;
                minormoonPrefixBox[k].Bordercolor = cbubordon;
            end

            if getlocalname(spacecraft[k+ spacecraftScroll]) == getlocalname(parentObj) then
                spacecraftPrefixBox[k].Textcolor = cbutexton;
                spacecraftPrefixBox[k].Bordercolor = cbubordon;
            end
        end

        -- We add the scrollButton on the second tick
        -- when bounds are getting updated.
        if solsysScroll_firstTickDone then
            if not(solsysScrollButtonDone) then
                solsysScrollButtonDone = true;
                makeScrollButton(planetScrollUpButton, planetScrollDownButton, planetScrollbarFrame);
                makeScrollButton(dwarfplanetScrollUpButton, dwarfplanetScrollDownButton, dwarfplanetScrollbarFrame);
                makeScrollButton(moonScrollUpButton, moonScrollDownButton, moonScrollbarFrame);
                makeScrollButton(minormoonScrollUpButton, minormoonScrollDownButton, minormoonScrollbarFrame);
                makeScrollButton(spacecraftScrollUpButton, spacecraftScrollDownButton, spacecraftScrollbarFrame);
            end
        else
            solsysScroll_firstTickDone = true;
        end
        if newwidth ~= width or newheight ~= height then
                    newwidth, newheight = width, height;       
                    solsysFrame:attach(screenBox, (width-solsysFrameWidth)/2, height-solsysFrameHeight-25, (width-solsysFrameWidth)/2, 25);                       
        end 
    end;
-- =============================================== окно для выбранного объекта
starBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)
    :bordercolor({0,1,0,.5}) 
    --:fillcolor({1,1,0,.2})
    :textfont(normalfont)
    :textcolor(cbutextoff)
    --:movetext(-3, 6)
    :movetext(0, 6+1)
    :text("")
    :movable(false)
    :active(true)
    :attach(solsysFrame, starLeftOffset+120, solsysFrameHeight-25, solsysFrameWidth-240, 10)

starBox.Action = (function()
    return
        function()
            celestia:select(star);
            parentObj = star;
            starName = nil;
        end
    end) ();

for k = 1, nChildrenMax do
    makeChildrenBox(planetBox, k, planetLeftOffset)
    makeChildrenBox(dwarfplanetBox, k, dwarfplanetLeftOffset)
    makeChildrenBox(moonBox, k, moonLeftOffset)
    makeChildrenBox(minormoonBox, k, minormoonLeftOffset)
    makeChildrenBox(spacecraftBox, k, spacecraftLeftOffset)

    makePrefixBox(planetPrefixBox, k, planetLeftOffset)
    makePrefixBox(dwarfplanetPrefixBox, k, dwarfplanetLeftOffset)
    makePrefixBox(moonPrefixBox, k, moonLeftOffset)
    makePrefixBox(minormoonPrefixBox, k, minormoonLeftOffset)
    makePrefixBox(spacecraftPrefixBox, k, spacecraftLeftOffset)
    makeStartBox(spacecraftStartBox, k, spacecraftLeftOffset)
    makeEndBox(spacecraftEndBox, k, spacecraftLeftOffset)

local kk = k;

planetBox[k].Action = (function()
    return
        function()
            celestia:select(planets[kk+planetScroll]);
        end
    end) ();

dwarfplanetBox[k].Action = (function()
    return
        function()
            celestia:select(dwarfplanets[kk+dwarfplanetScroll]);
        end
    end) ();

moonBox[k].Action = (function()
    return
        function()
            celestia:select(moons[kk+moonScroll]);
        end
    end) ();

minormoonBox[k].Action = (function()
    return
        function()
            celestia:select(minormoons[kk+minormoonScroll]);
        end
    end) ();

spacecraftBox[k].Action = (function()
    return
        function()
            celestia:select(spacecraft[kk+spacecraftScroll]);
        end
    end) ();

planetPrefixBox[k].Action = (function()
    return
        function()
            parentObj = planets[kk+planetScroll];
            parentObjChanged = true;
        end
    end) ();

dwarfplanetPrefixBox[k].Action = (function()
    return
        function()
            parentObj = dwarfplanets[kk+dwarfplanetScroll];
            parentObjChanged = true;
        end
    end) ();

moonPrefixBox[k].Action = (function()
    return
        function()
            parentObj = moons[kk+moonScroll];
            parentObjChanged = true;
        end
    end) ();

minormoonPrefixBox[k].Action = (function()
    return
        function()
            parentObj = minormoons[kk+minormoonScroll];
            parentObjChanged = true;
        end
    end) ();

spacecraftPrefixBox[k].Action = (function()
    return
        function()
            parentObj = spacecraft[kk+spacecraftScroll];
            parentObjChanged = true;
        end
    end) ();

spacecraftStartBox[k].Action = (function()
    return
        function()
            local startdate = spacecraft[kk+spacecraftScroll]:getinfo().lifespanStart;
            celestia:settime(startdate);
        end
    end) ();

spacecraftEndBox[k].Action = (function()
    return
        function()
            local enddate = spacecraft[kk+spacecraftScroll]:getinfo().lifespanEnd;
            celestia:settime(enddate);
        end
    end) ();

end

planetEtcBox = makeEtcBox(planetLeftOffset)
dwarfplanetEtcBox = makeEtcBox(dwarfplanetLeftOffset)
moonEtcBox = makeEtcBox(moonLeftOffset)
minormoonEtcBox = makeEtcBox(minormoonLeftOffset)
spacecraftEtcBox = makeEtcBox(spacecraftLeftOffset)
planetScrollbarFrame = makeChildScrollbarFrame(planetLeftOffset);
dwarfplanetScrollbarFrame = makeChildScrollbarFrame(dwarfplanetLeftOffset);

moonScrollbarFrame = makeChildScrollbarFrame(moonLeftOffset);
minormoonScrollbarFrame = makeChildScrollbarFrame(minormoonLeftOffset);
spacecraftScrollbarFrame = makeChildScrollbarFrame(spacecraftLeftOffset);

planetScrollbarBox = makeChildScrollbarBox(planetScrollbarFrame, planetLeftOffset, nPlanets);
dwarfplanetScrollbarBox = makeChildScrollbarBox(dwarfplanetScrollbarFrame, dwarfplanetLeftOffset, nDwarfPlanets);
moonScrollbarBox = makeChildScrollbarBox(moonScrollbarFrame, moonLeftOffset, nMoons);
minormoonScrollbarBox = makeChildScrollbarBox(minormoonScrollbarFrame, minormoonLeftOffset, nMinorMoons);
spacecraftScrollbarBox = makeChildScrollbarBox(spacecraftScrollbarFrame, spacecraftLeftOffset, nSpacecraft);

planetScrollbarFrame.Customdraw =
    function(this)
        planetScroll = refreshChildScrollbarFrame(planetScrollbarFrame, planetScrollbarBox);
    end;

planetScrollbarBox.Customdraw =
    function(this)
        refreshChildScrollbarBox(planetScrollbarFrame,planetScrollbarBox, planetScroll, planetScrollbarBoxH, nPlanets)
    end;

dwarfplanetScrollbarFrame.Customdraw =
    function(this)
        dwarfplanetScroll = refreshChildScrollbarFrame(dwarfplanetScrollbarFrame, dwarfplanetScrollbarBox);
    end;

dwarfplanetScrollbarBox.Customdraw =
    function(this)
        refreshChildScrollbarBox(dwarfplanetScrollbarFrame,dwarfplanetScrollbarBox, dwarfplanetScroll, dwarfplanetScrollbarBoxH, nDwarfPlanets)
    end;

moonScrollbarFrame.Customdraw =
    function(this)
        moonScroll = refreshChildScrollbarFrame(moonScrollbarFrame, moonScrollbarBox);
    end;

moonScrollbarBox.Customdraw =
    function(this)
        refreshChildScrollbarBox(moonScrollbarFrame, moonScrollbarBox, moonScroll, moonScrollbarBoxH, nMoons)
    end;

minormoonScrollbarFrame.Customdraw =
    function(this)
        minormoonScroll = refreshChildScrollbarFrame(minormoonScrollbarFrame, minormoonScrollbarBox);
    end;

minormoonScrollbarBox.Customdraw =
    function(this)
        refreshChildScrollbarBox(minormoonScrollbarFrame, minormoonScrollbarBox, minormoonScroll, minormoonScrollbarBoxH, nMinorMoons)
    end;

spacecraftScrollbarFrame.Customdraw =
    function(this)
        spacecraftScroll = refreshChildScrollbarFrame(spacecraftScrollbarFrame, spacecraftScrollbarBox);
    end;

spacecraftScrollbarBox.Customdraw =
    function(this)
        refreshChildScrollbarBox(spacecraftScrollbarFrame, spacecraftScrollbarBox, spacecraftScroll, spacecraftScrollbarBoxH, nSpacecraft)
    end;
-- =================================
star1Box = CXBox:new()
    :init(0, 0, 0, 0) 
    :bordercolor(cbubordoff)  
    :fillcolor({1,1,0,.2})
    :textfont(normalfont)
    :textcolor(cbutextoff)
    :movetext(0, 8)                                    
    :text(_("Find Sol"))
    :movable(true)
    :active(true)   
    :attach(solsysFrame, 260, solsysFrameHeight-25, 380, 10)  

star1Box.Action = (function()
    return
        function()
            local obs = celestia:getobserver();
            local obj = celestia:find("Sol");
            celestia:select(obj);
            obs:follow(obj);          
            obs:center(obj);            
        end
    end) ();
---------------------------
gotoSelButton= CXBox:new()
    :init(0, 0, 0, 0) 
    :bordercolor(cbubordoff)  
    :fillcolor({0,0,1,0.5})
    :textfont(normalfont)
    :textcolor(cbutextoff)
    :movetext(0, 6+1)                              
    :text(_("Goto"))
    :movable(true)
    :active(true)  
    :attach(solsysFrame, 460+300-150, solsysFrameHeight-25, 20, 10)  

gotoSelButton.Action = (function()
    return
         function()
            local obs = celestia:getobserver();
            local sel = celestia:getselection();
            local obj = parentObj             
            if not(empty(obj)) then
                celestia:select(obj);
                obs:follow(obj);
                obs:goto(obj);
            end
        end
    end) ();

--[[
--======================================
-- ViDiBaBox.lua
-- Author - Vincent Giangiulio a.k.a Vincent
-- Edit by Joёl a.k.a Jogad 
-- Integration by Lygin Gennady a.k.a Croc 21.12.2013 и  28.08.2016
--======================================
require "config"  -- new Jogad (здесь и далее 28.08.2016)
require "locale";
-- require "closeButton";
require "scrollButton";

----------------------------------------------
-- Set initial values
----------------------------------------------
local newwidth1, newheight1;
local locLabelBoxWidth, locLabelBoxHeight = 100, 20;
local featureSizeBoxWidth, featureSizeBoxHeight = 146, 43;

-- Set the height of the ViDiBa window.
local addon_list_nlines = addon_list_nlines or 4; -- value from config.lua -- new Jogad
local nViDiBa = table.getn(ViDiBa);
local nViDiBaLinesMax = math.max(4, math.min(addon_list_nlines, nViDiBa)); -- edit Jogad
local ViDiBaFrameWidth, ViDiBaFrameHeight = 250, 110 + (20 * nViDiBaLinesMax); -- edit Jogad
local ViDiBaScrollbarFrameW, ViDiBaScrollbarFrameH = 9, ViDiBaFrameHeight  - 110 - 20; -- edit Jogad

local nViDiBaLines = nViDiBa; -- edit Jogad
local ViDiBaScroll = 0; -- edit Jogad
local ViDiBaScrollCoeff = 6; -- edit Jogad
local ViDiBaScrollbarBoxH = 0; -- edit Jogad

local minFeatureSize = celestia:getminfeaturesize();

local featureSizeSliderFrameW = 130;  -- edit Jogad
local featureSizeSliderBoxW = 12;  -- edit Jogad
local featureSizeSliderMargin = 3;  -- edit Jogad
local featureSizePosMax = featureSizeSliderFrameW - featureSizeSliderBoxW - (2 * featureSizeSliderMargin); -- edit Jogad
local featureSizePos = math.ceil(minFeatureSize / 100 * featureSizePosMax); -- edit Jogad

--local addLeftOffset = 20;  -- edit Jogad
local addLeftOffset = 10; -- edit Croc

local addBox = {}; -- edit Jogad
local addCheckBox = {}; -- edit Jogad
local addVisible = {}; -- edit Jogad
local addLabelColor = {}; -- edit Jogad

-- Disable all Location types.
local obs = celestia:getobserver();
local locFlags = obs:getlocationflags();
for k, v in pairs(locFlags) do
    locFlags[k] = false;
    obs:setlocationflags(locFlags);
end

local firstTickDone = false   -- edit Jogad
local closeButtonDone = false   -- edit Jogad

-- Enable Location labels.
celestia:setlabelflags{locations = true}

----------------------------------------------
-- Define local functions
----------------------------------------------
local setCheckBoxText = function(box, flag)
    if flag then
        box.Text = "v";
    else
        box.Text = "";
    end
end

local setAddCheckBoxText = function(box, k, flag)
    if flag then
        box[k].Text = "v";
    else
        box[k].Text = "";
    end
end

----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------

ViDiBaFrame = CXBox:new()
    :init(0, 0, 0, 0)
    :fillcolor(csetbofill)
    :bordercolor(cbubordoff)
    :movable(true)
    :visible(false)
    :attach(screenBox, width/2+1, height-ViDiBaFrameHeight-2, width/2-ViDiBaFrameWidth-3, 2)

setViDiBaButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0.2,0.8,1,.6})
    :fillcolor(cFrameFill)
    :textfont(smallfont)
    :textcolor({0.2,0.8,1,1})
    :textpos("left")
    :movetext(0, 10)   
    :text(_(" ViDiBa"))
    :movable(false)
    :active(true)   
    :attach(solarSystemBox, -100+3+50+3, 0, 50-1, 0)

setViDiBaButton.Action = function()
	ViDiBaFrame.Visible = not(ViDiBaFrame.Visible);
	if ViDiBaFrame.Visible then
		ViDiBaFrame:orderfront();
                setViDiBaButton.Bordercolor = {1,0,0,.6};
	--	setViDiBaButton.Fillcolor = {cbubordon[1]*2/3, cbubordon[2]*2/3, cbubordon[3]*2/3, cbubordon[4]/2};
	else
		setViDiBaButton.Bordercolor = {0.2,0.8,1,.6};   
	--	setViDiBaButton.Fillcolor = cFrameFill;   
	end
end

ViDiBaFrame.Customdraw = function(this)
            textlayout:setfont(normalfont);
            textlayout:setfontcolor({0,1,0,0.8});  
            textlayout:setpos(this.lb+addLeftOffset, this.tb-20);
            textlayout:println(_("ViDiBa visibility:"));

        if firstTickDone then
            if not(closeButtonDone) then
                closeButtonDone = true;
                makeCloseButton(ViDiBaCloseButton, ViDiBaFrame, setViDiBaButton);
            end
        else
            firstTickDone = true;
        end
        if newwidth1 ~= width or newheight1 ~= height then
            newwidth1, newheight1 = width, height;        
            ViDiBaFrame:attach(screenBox, width/2+1, height-ViDiBaFrameHeight-10, width/2-ViDiBaFrameWidth-3, 10)
        end
    end;
---------------------------------------

local makeAddBox = function(k, v, leftOffset, color)

addBox[k] = CXBox:new()
        :init(0, 0, 0, 0)
        --:bordercolor(cbubordoff)
        --:fillcolor({1,1,0,.2})
        :textfont(normalfont)
        :textcolor(color)
        :movetext(0, 8)    
        :text(_(v))
        :movable(false)
        :attach(ViDiBaFrame, leftOffset+15, ViDiBaFrameHeight-25-(20*k), ViDiBaFrameWidth-leftOffset-100, 14+(20*k))
end

local makeAddCheckBox = function(k, leftOffset)
    addCheckBox[k] = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(cbubordoff)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(ViDiBaFrame, leftOffset, ViDiBaFrameHeight-25-(20*k), ViDiBaFrameWidth-leftOffset-11+2, 14+(20*k))

end

for k, v in ipairs(ViDiBa) do

    if pcall( function() require (v) end ) then
        require(v)

        addLabelColor[k] = cbutextoff;
        if _G[v].labelcolor then
            addLabelColor[k] = _G[v].labelcolor;
        end

        makeAddBox(k, string.gsub(v, "_", " "), addLeftOffset, addLabelColor[k]);
        makeAddCheckBox(k, addLeftOffset);

        local vv, kk = v, k;

        if _G[v].objects then
            addVisible[k] = celestia:find(_G[v].objects[1]):visible();
        else
            addVisible[k] = false;
        end

        addBox[k].Visible = k <= nViDiBaLinesMax;
        addCheckBox[k].Visible = k <= nViDiBaLinesMax;

        addCheckBox[k].Customdraw =
            function(this)
                setAddCheckBoxText(addCheckBox, kk, addVisible[k + ViDiBaScroll]);
                addBox[k].Text = _(string.gsub(ViDiBa[k + ViDiBaScroll], "_", " "));
                addBox[k].Textcolor = addLabelColor[k + ViDiBaScroll];
            end

            addCheckBox[k].Action = (function()
                return
                    function()
                        local vViDiBaScroll = ViDiBa[k + ViDiBaScroll];
                        addVisible[k + ViDiBaScroll] = not(addVisible[k + ViDiBaScroll]);
                        if _G[vViDiBaScroll].objects then
                            for n, obj in pairs(_G[vViDiBaScroll].objects) do
                                local addObj = celestia:find(obj);
                                addObj:setvisible(not addObj:visible());
                            end
                        end

                        if _G[vViDiBaScroll].locationtypes then
                            for n, loctype in pairs(_G[vViDiBaScroll].locationtypes) do
                                addLocType = string.lower(loctype);
                                local obs = celestia:getobserver();
                                local locFlags = obs:getlocationflags();
                                if locFlags[addLocType] ~= addVisible[k + ViDiBaScroll] then
                                    locFlags[addLocType] = addVisible[k + ViDiBaScroll];
                                    obs:setlocationflags(locFlags);
                                end
                            end
                        end

                        if _G[vViDiBaScroll].script then
                            local scriptfile = "../ViDiBa/"..vViDiBaScroll.."/".._G[vViDiBaScroll].script;
                            celestia:runscript(scriptfile);
                            if _G[vViDiBaScroll].objects then
                                addVisible[k + ViDiBaScroll] = celestia:find(_G[vViDiBaScroll].objects[1]):visible();
                                setAddCheckBoxText(addCheckBox, kk, addVisible[k + ViDiBaScroll]);
                            end
                        end
                    end
                end) ();
    end
end

ViDiBaScrollbarFrame = CXBox:new()
    :bordercolor(cbubordoff) -- 19.03.2017
    --:fillcolor({1, 0, 0, 1})
    :visible(nViDiBaLines > nViDiBaLinesMax)
    :attach(ViDiBaFrame, ViDiBaFrameWidth-ViDiBaScrollbarFrameW-3+2, ViDiBaFrameHeight-40-ViDiBaScrollbarFrameH, 3, 40);

ViDiBaScrollbarFrame.Customdraw =
    function(this)
        -- We add the scrollButton on the second tick
        -- when bounds are getting updated.
        if ViDiBaScroll_firstTickDone then
            if not(ViDiBaScrollButtonDone) then
                ViDiBaScrollButtonDone = true;
                makeScrollButton(ViDiBaScrollUpButton, ViDiBaScrollDownButton, ViDiBaScrollbarFrame);
            end
        else
            ViDiBaScroll_firstTickDone = true;
        end

        local lb3,bb3,rb3,tb3 =ViDiBaScrollbarFrame:bounds();
        local lb4,bb4,rb4,tb4 = ViDiBaScrollbarBox:bounds();
        ViDiBaScroll = math_round((tb3-tb4-3)/ViDiBaScrollCoeff);
        ViDiBaScrollbarFrame:attach(ViDiBaFrame, ViDiBaFrameWidth-ViDiBaScrollbarFrameW-3+2, ViDiBaFrameHeight-40-ViDiBaScrollbarFrameH, 3, 40);
    end

ViDiBaScrollbarBox = CXBox:new()
    :bordercolor(cbubordoff)
    :fillcolor(cscrollfill)
    :visible(true)
    :movable(true)
    :attach(ViDiBaScrollbarFrame, 0, (nViDiBaLines-nViDiBaLinesMax)*ViDiBaScrollCoeff, 0, 3)
]]
--[[
-- ==============================================
-- legBox.lua   = Ter
-- by Gennady Lygin a.k.a Croc & Patrick Ziegler a.k.a. Gironde 09.03.2017    
-- ==============================================
require "locale";
-- require "closeButton";
----------------------------------------------
-- Set initial values
----------------------------------------------
local newwidth2, newheight2;
local gn=3
local gg=gn-1
local g=15
local legFrameWidth, legFrameHeight = 15*51+14*1+3, 12*gn+5; -- 782, 42 
----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------
legButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0.2,0.8,1,.6})
    :fillcolor(cFrameFill)
   -- :textfont(normalfont)
    :textfont(smallfont)
    :textcolor({0.2,0.8,1,1})
    :textpos("left")
    :movetext(1, 10)   
    :text(_("Legend"))
    :movable(false)
    :active(true)   
    :attach(solarSystemBox, -100+3+50+1, 0, 0, 0)

legButton.Action = function()
	legFrame.Visible = not(legFrame.Visible);
	if legFrame.Visible then
		legFrame:orderfront();
                legButton.Bordercolor = {1,0,0,.6};
	else
		legButton.Bordercolor = {0.2,0.8,1,.6}; 
   	end
end

legFrame = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :fillcolor(color82)
    :movable(true)
    :clickable(false)    
    :visible(false)
    :attach(screenBox, (width-legFrameWidth)/2, height-legFrameHeight-2 ,(width-legFrameWidth)/2, 2) -- 397, 760, 397, 2

 legFrame.Customdraw = function(this)  

            if  leg_firstTickDone then
              if not(legCloseButtonDone) then
                 legCloseButtonDone = true;
                 makeCloseButton(legCloseButton, legFrame,  legButton);
              end
            else
                leg_firstTickDone = true;            
            end     
            if newwidth2 ~= width or newheight2 ~= height then
                    newwidth2, newheight2 = width, height;
                    legFrame:attach(screenBox, (width-legFrameWidth)/2, height-legFrameHeight-9 ,(width-legFrameWidth)/2, 9)
            end

	    textlayout:setfont(normalfont);
            textlayout:setfontcolor({0.2,0.8,1,1});        
	    textlayout:setpos(this.lb+600, this.tb - 11);       
            textlayout:println(_("Conventions designations:"));
end;
--=================
leg11Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    :textfont(smallfont)
    :textcolor(color1)
    :textpos("center") 
    :movetext(-2, 9)
    :text(_("N. stars"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2, 14*(gg-0), (51+1)*14+2, 2+14*0-0)

leg12Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :textfont(smallfont)
    :textcolor(color0)
    :textpos("center") 
    :movetext(-2, 9)
    :text(_("Zodiac"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2, 14*(gg-1)+1, (51+1)*14+2, 2+14*1-1)

leg13Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :textfont(smallfont)
    :textcolor(color2)
    :textpos("center") 
    :movetext(-2, 9)
    :text(_("S. stars"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2, 14*(gg-2)+2, (51+1)*14+2, 2+14*2-2)
--=========================
leg21Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color01)
    :textfont(smallfont)  
    :textcolor(color7)   
    :textpos("center")
    :movetext(-3, 9)
    :text(_("PLANET"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*1, 14*(gg-0), (51+1)*13+2, 2+14*0-0)

leg22Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color02)
    :textfont(smallfont)
    :textcolor(color2)   
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Moon"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*1, 14*(gg-1)+1, (51+1)*13+2, 2+14*1-1)

leg23Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color03)
    :textfont(smallfont)
    :textcolor(color6)  
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("M.Moon"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*1, 14*(gg-2)+2, (51+1)*13+2, 2+14*2-2)
-- =================
leg31Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color81)
    :textfont(smallfont) 
    :textcolor(color7)  
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Dwarf PL."))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*2, 14*(gg-0), (51+1)*12+2, 2+14*0-0)

leg32Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color82)
    :textfont(smallfont) 
    :textcolor(color3)
    :textpos("center") 
    :movetext(0, 9)
    :text(_("Trojan"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*2, 14*(gg-1)+1, (51+1)*12+2, 2+14*1-1)

leg33Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color83)
    :textfont(smallfont)  
    :textcolor(color1) 
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Satellite"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*2, 14*(gg-2)+2, (51+1)*12+2, 2+14*2-2)
-- =================
leg41Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :bordercolor(cbubordoff)
    :fillcolor(color91) 
    :textfont(smallfont)  
    :textcolor(color7)  
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Plutoid"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*3, 14*(gg-0), (51+1)*11+2, 2+14*0-0)

leg42Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color92) 
    :textfont(smallfont)  
    :textcolor(color4)
    :textpos("center") 
    :movetext(-3, 9)
    :text(_("Sednoid"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*3, 14*(gg-1)+1, (51+1)*11+2, 2+14*1-1)

leg43Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color93)
    :textfont(smallfont)  
    :textcolor(color9)   
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("> 400 km."))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*3, 14*(gg-2)+2, (51+1)*11+2, 2+14*2-2)
-- =================
leg51Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color11)
    :textfont(smallfont)  
    :textcolor(color9) 
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Atira"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*4, 14*(gg-0), (51+1)*10+2, 2+14*0-0)

leg52Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color12)
    :textfont(smallfont)  
    :textcolor(color9)   
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Aton"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*4, 14*(gg-1)+1, (51+1)*10+2, 2+14*1-1)

leg53Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color13)
    :textfont(smallfont)  
    :textcolor(color9) 
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Apollo"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*4, 14*(gg-2)+2, (51+1)*10+2, 2+14*2-2)
-- =================
leg61Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color21)
    :textfont(smallfont)  
    :textcolor(color9)   
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Amurs"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*5, 14*(gg-0), (51+1)*9+2, 2+14*0-0)

leg62Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color22)
    :textfont(smallfont)  
    :textcolor(color9)   
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("M.Cross."))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*5, 14*(gg-1)+1, (51+1)*9+2, 2+14*1-1)

leg63Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color23)
    :textfont(smallfont)  
    :textcolor(color9) 
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Centaurs"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*5, 14*(gg-2)+2, (51+1)*9+2, 2+14*2-2)
-- =================
leg71Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color31)
    :textfont(smallfont)  
    :textcolor(color9)   
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("AB intern."))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*6, 14*(gg-0), (51+1)*8+2, 2+14*0-0)

leg72Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color32)
    :textfont(smallfont)  
    :textcolor(color9)   
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("AB basic"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*6, 14*(gg-1)+1, (51+1)*8+2, 2+14*1-1)

leg73Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color33)
    :textfont(smallfont)  
    :textcolor(color9) 
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("AB outer"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*6, 14*(gg-2)+2, (51+1)*8+2, 2+14*2-2)
-- =================
leg81Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color41)
    :textfont(smallfont)  
    :textcolor(color9)   
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Cubewan."))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*7, 14*(gg-0), (51+1)*7+2, 2+14*0-0)

leg82Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color42)
    :textfont(smallfont)  
    :textcolor(color9)   
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Plutino"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*7, 14*(gg-1)+1, (51+1)*7+2, 2+14*1-1)

leg83Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color43)
    :textfont(smallfont)  
    :textcolor(color9) 
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Tutino"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*7, 14*(gg-2)+2, (51+1)*7+2, 2+14*2-2)
-- =================
leg91Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color51)
    :textfont(smallfont)  
    :textcolor(color9) 
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("KB oth.res."))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*8, 14*(gg-0), (51+1)*6+2, 2+14*0-0)

leg92Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color52)
    :textfont(smallfont)  
    :textcolor(color9)   
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("KB other"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*8, 14*(gg-1)+1, (51+1)*6+2, 2+14*1-1)

leg93Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color53)
    :textfont(smallfont) 
    :textcolor(color9)  
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Kuiper B."))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*8, 14*(gg-2)+2, (51+1)*6+2, 2+14*2-2)
-- =================
leg101Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color61)
    :textfont(smallfont)  
    :textcolor(color9)   
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("SD oth.res."))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*9, 14*(gg-0), (51+1)*5+2, 2+14*0-0)

leg102Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color62)
    :textfont(smallfont)  
    :textcolor(color9) 
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("SD other"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*9, 14*(gg-1)+1, (51+1)*5+2, 2+14*1-1)

leg103Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color63)
    :textfont(smallfont)  
    :textcolor(color9) 
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Scat. disc"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*9, 14*(gg-2)+2, (51+1)*5+2, 2+14*2-2)
-- =================
leg111Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color71)
    :textfont(smallfont)  
    :textcolor(color9) 
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Damokloid"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*10, 14*(gg-0), (51+1)*4+2, 2+14*0-0)

leg112Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color72)
    :textfont(smallfont)  
    :textcolor(color9) 
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Detached"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*10, 14*(gg-1)+1, (51+1)*4+2, 2+14*1-1)

leg113Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :fillcolor(color73)
    :textfont(smallfont)  
    :textcolor(color9)
    :textpos("center")    
    :movetext(-3, 9)
    :text(_("Oort Cloud"))
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*10, 14*(gg-2)+2, (51+1)*4+2, 2+14*2-2)
-- ================
leg122Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :textfont(smallfont) 
    :textcolor(color1)
    :textpos("left")  
    :movetext(-3, 9) 
    :text(_("+ - there is satellite")) 
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*11, 14*(gg-1)+1, (51+1)*2+2, 2+14*1-1)

leg123Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :textfont(smallfont) 
    :textcolor(color1)
    :textpos("left")  
    :movetext(-3, 9) 
    :text(_("# - No object in base")) 
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*11, 14*(gg-2)+2, (51+1)*2+2, 2+14*2-2)
-- =================
leg142Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :textfont(smallfont) 
    :textcolor(color1)
    :textpos("left")  
    :movetext(-3, 9) 
    :text(_(" ! - Non-stand. model"))  
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*13, 14*(gg-1)+1, (51+1)*0+2, 2+14*1-1)

leg143Box = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff) 
    :textfont(smallfont) 
    :textcolor({0.1,0.5,0,1})
    :textpos("left")  
    :movetext(-3, 9) 
    :text(_("Specific mesh 3D"))  
    :movable(false)
    :active(false) 
    :attach(legFrame, 2+(51+1)*13, 14*(gg-2)+2, (51+1)*0+2, 2+14*2-2)


--======================================
-- RenderLocsBox
-- Author - Patrick Ziegler a.k.a. Gironde 10.03.2017 
-- assisted by Croc
-- Integration by Lygin Gennady a.k.a Croc 12.03.2017
--======================================
require "config"  
require "locale";
-- require "closeButton";

-----------------------------------
--  Set initial values
-----------------------------------
local newwidth3, newheight3
local locsLabelBoxWidth, locsLabelBoxHeight = 100, 20
local nlines = 15
-- local addwidth = 150 -- offset for each column
local addwidth = 170 -- offset for each column
-- local renderLocsFrameWidth, renderLocsFrameHeight = 150+addwidth, (20 * nlines)+40
local renderLocsFrameWidth, renderLocsFrameHeight = 170+addwidth, (20 * nlines)+40

local renderlocsLeftMargin = 10
local obs = celestia:getobserver()
local tableflag = obs:getlocationflags()
-- local featckeckcolor = color2 -- color2 or cbubordoff
local featckeckcolor = cbubordoff

-- Enabled Location labels --------------------------------------
celestia:setlabelflags{locations = true}	-- enabled name of locations


--------------------------------------------------------------------------------------------------------------------------
--                                          Set up and Draw the boxes
--------------------------------------------------------------------------------------------------------------------------

-- ================= Button's command frame =======================
renderLocsBox = CXBox:new() 
    :init(0, 0, 0, 0)
    --:bordercolor({0.2,0.8,1,.6})
    :fillcolor(cFrameFill)
    :movable(false)
    :attach(screenBox, width, height, 0, 0);

----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------

renderLocsButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor({0.2,0.8,1,.6})
    :fillcolor(cFrameFill)
   -- :textfont(normalfont)
    :textfont(smallfont)
    :textcolor({0.2,0.8,1,1})
    :textpos("left")
    :movetext(-3, 10)   
    :text(_("Location"))
    :movable(false)
    :active(true)   
    :attach(solarSystemBox, -44, 0, 96, 0)
	
renderLocsButton.Action = (function() -- display or not the frame renderLocs
    return
        function()
            renderLocsFrame.Visible = not(renderLocsFrame.Visible);
                    if renderLocsFrame.Visible then
                        renderLocsFrame:orderfront();
                        renderLocsButton.Bordercolor = {1,0,0,.6}
                    else
                        renderLocsButton.Bordercolor = {0.2,0.8,1,.6}
                    end
-------------------------------------------------------------------------------------------------------
       end
  end) ();
  
renderLocsButtonCheck = CXBox:new()
        :init(0, 0, 0, 0)
        :textfont(smallfont)
        :textcolor({1,0,0,1})
        :textpos("left")
        :movetext(-32, 9)
        :text("")
        :movable(false)
        :active(true)  
        :attach(renderLocsBox, 20, 0, 0, 0)
  
-- ======================= draw the frame========================================
renderLocsFrame = CXBox:new()
    :init(0, 0, 0, 0)
    :fillcolor(csetbofill)
    :bordercolor(cbubordoff)
    :textfont(titlefont)
    :textcolor({0.8, 0.8, 0.8, 0.9})
    :textpos("center")
    :text(_(""))
    :movetext(0, 5)
    :movable(true)
    :visible(false)
    :attach(screenBox, (width-renderLocsFrameWidth)/2+100, height-renderLocsFrameHeight-2, (width-renderLocsFrameWidth)/2-100, 2)
	
renderLocsFrame.Customdraw = 
    function(this)

            textlayout:setfont(normalfont);
            textlayout:setfontcolor({0,1,0,0.8});
            textlayout:setpos(this.lb+renderlocsLeftMargin, this.tb-20);
            textlayout:println(_("LOCATIONS: FEATURES COMPLEMENTARY")); 

        if renderLocs_firstTickDone then
            if not(renderLocscloseButtonDone) then
                renderLocscloseButtonDone = true;
                makeCloseButton(renderLocsCloseButton, renderLocsFrame, renderLocsButton);
            end
        else
            renderLocs_firstTickDone = true;
        end

        if newwidth3 ~= width or newheight3 ~= height then
            newwidth3, newheight3 = width, height
            renderLocsFrame:attach(screenBox, (width-renderLocsFrameWidth)/2-118, height-renderLocsFrameHeight-10, (width-renderLocsFrameWidth)/2+118, 10)
        end
    end;
--================================= 1e colonne ==========================================
----------------------------------------------------------------------------------------
CityLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("City"))
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-1), locsLabelBoxWidth+72, 20+19*1);

CityCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(CityLabelBox, 2, 10, 147, 4)

 CityCheckBox.Customdraw =
    function(this)
                if tableflag.city then							-- state of city in the table 
                    CityCheckBox.Text = "v"
                else
                    CityCheckBox.Text = ""
                end
    end;
       
CityCheckBox.Action = (function()
        return
            function()
                if tableflag.city then 
                    CityCheckBox.Text = ""
                    tableflag.city = false
                else
                    CityCheckBox.Text = "v"
					tableflag.city = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
ObservatoryLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Observatory"))
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-2), locsLabelBoxWidth+72, 20+19*2);

ObservatoryCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(ObservatoryLabelBox, 2, 10, 147, 4)

ObservatoryCheckBox.Customdraw =
    function(this)
                if tableflag.observatory then							-- state in the table 
                    ObservatoryCheckBox.Text = "v"
                else
                    ObservatoryCheckBox.Text = ""
                end
    end;
       
ObservatoryCheckBox.Action = (function()
        return
            function()
                if tableflag.observatory then
                    ObservatoryCheckBox.Text = ""
                    tableflag.observatory = false
                else
                    ObservatoryCheckBox.Text = "v"
					tableflag.observatory = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
BoundaryLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Boundary (see Addon)")) -- fronti鳥
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-3), locsLabelBoxWidth+72, 20+19*3);

BoundaryCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(cbubordoff)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(BoundaryLabelBox, 2, 10, 147, 4)

--BoundaryCheckBox.Customdraw =
--    function(this)
--                if tableflag.boundary then							-- state in the table 
--                    BoundaryCheckBox.Text = "v"
--                else

--                    BoundaryCheckBox.Text = ""
--                end

--    end;
       
--BoundaryCheckBox.Action = (function()
--        return
--            function()
--                if tableflag.boundary then
--                    BoundaryCheckBox.Text = ""
--                    tableflag.boundary = false
--                else

--                    BoundaryCheckBox.Text = "v"
--					tableflag.boundary = true
--                end

--                obs:setlocationflags(tableflag)
--            end

--        end) ();

----------------------------------------------------------------------------------------
LandingsiteLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Landingsite (LF)")) -- Landing site for the Moon; it exist "landing site" for other
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-4), locsLabelBoxWidth+72, 20+19*4);

LandingsiteCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(LandingsiteLabelBox, 2, 10, 147, 4)

LandingsiteCheckBox.Customdraw =
    function(this)
                if tableflag.landingsite then							-- state in the table 
                    LandingsiteCheckBox.Text = "v"
                else
                    LandingsiteCheckBox.Text = ""
                end
    end;
       
LandingsiteCheckBox.Action = (function()
        return
            function()
                if tableflag.landingsite then
                    LandingsiteCheckBox.Text = ""
                    tableflag.landingsite = false
                else
                    LandingsiteCheckBox.Text = "v"
					tableflag.landingsite = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
CraterLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Crater (AA)")) -- Impact crater
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-5), locsLabelBoxWidth+72, 20+19*5);

CraterCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(CraterLabelBox, 2, 10, 147, 4)

CraterCheckBox.Customdraw =
    function(this)
                if tableflag.crater then							-- state in the table 
                    CraterCheckBox.Text = "v"
                else
                    CraterCheckBox.Text = ""
                end
    end;
       
CraterCheckBox.Action = (function()
        return
            function()
                if tableflag.crater then
                    CraterCheckBox.Text = ""
                    tableflag.crater = false
                else
                    CraterCheckBox.Text = "v"
					tableflag.crater = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
VallisLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Vallis (VA)")) -- Valley
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-6), locsLabelBoxWidth+72, 20+19*6);

VallisCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(VallisLabelBox, 2, 10, 147, 4)

VallisCheckBox.Customdraw =
    function(this)
                if tableflag.vallis then							-- state in the table 
                    VallisCheckBox.Text = "v"
                else
                    VallisCheckBox.Text = ""
                end
    end;
       
VallisCheckBox.Action = (function()
        return
            function()
                if tableflag.vallis then
                    VallisCheckBox.Text = ""
                    tableflag.vallis = false
                else
                    VallisCheckBox.Text = "v"
					tableflag.vallis = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
MonsLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Mons (MO)")) -- Mountain
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-7), locsLabelBoxWidth+72, 20+19*7);

MonsCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(MonsLabelBox, 2, 10, 147, 4)

MonsCheckBox.Customdraw =
    function(this)
                if tableflag.mons then							-- state in the table 
                    MonsCheckBox.Text = "v"
                else
                    MonsCheckBox.Text = ""
                end
    end;
       
MonsCheckBox.Action = (function()
        return
            function()
                if tableflag.mons then
                    MonsCheckBox.Text = ""
                    tableflag.mons = false
                else
                    MonsCheckBox.Text = "v"
					tableflag.mons = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
PlanumLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Planum (PM)")) -- Plateau
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-8), locsLabelBoxWidth+72, 20+19*8);

PlanumCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(PlanumLabelBox, 2, 10, 147, 4)

PlanumCheckBox.Customdraw =
    function(this)
                if tableflag.planum then							-- state in the table 
                    PlanumCheckBox.Text = "v"
                else
                    PlanumCheckBox.Text = ""
                end
    end;
       
PlanumCheckBox.Action = (function()
        return
            function()
                if tableflag.planum then
                    PlanumCheckBox.Text = ""
                    tableflag.planum = false
                else
                    PlanumCheckBox.Text = "v"
					tableflag.planum = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
MareLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Mare (ME)")) -- Sea, Ocean
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-9), locsLabelBoxWidth+72, 20+19*9);

MareCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(MareLabelBox, 2, 10, 147, 4)

MareCheckBox.Customdraw =
    function(this)
                if tableflag.mare then							-- state in the table 
                    MareCheckBox.Text = "v"
                else
                    MareCheckBox.Text = ""
                end
    end;
       
MareCheckBox.Action = (function()
        return
            function()
                if tableflag.mare then
                    MareCheckBox.Text = ""
                    tableflag.mare = false
                else
                    MareCheckBox.Text = "v"
					tableflag.mare = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
ChasmaLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Chasma (CM)")) -- Exogeology: canyon
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-10), locsLabelBoxWidth+72, 20+19*10);

ChasmaCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(ChasmaLabelBox, 2, 10, 147, 4)

ChasmaCheckBox.Customdraw =
    function(this)
                if tableflag.chasma then							-- state in the table 
                    ChasmaCheckBox.Text = "v"
                else
                    ChasmaCheckBox.Text = ""
                end
    end;
       
ChasmaCheckBox.Action = (function()
        return
            function()
                if tableflag.chasma then
                    ChasmaCheckBox.Text = ""
                    tableflag.chasma = false
                else
                    ChasmaCheckBox.Text = "v"
					tableflag.chasma = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();--        

----------------------------------------------------------------------------------------
PateraLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Caldera (PE)")) -- Exogeology: alien volcanoes, caldera, craters
    :movable(false)
    :active(true) 
	-- 2eme colonne at left +150, at right -150
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-11), locsLabelBoxWidth+72, 20+19*11);

PateraCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(PateraLabelBox, 2, 10, 147, 4)

PateraCheckBox.Customdraw =
    function(this)
                if tableflag.patera then							-- state in the table 
                    PateraCheckBox.Text = "v"
                else
                    PateraCheckBox.Text = ""
                end
    end;
       
PateraCheckBox.Action = (function()
        return
            function()
                if tableflag.patera then
                    PateraCheckBox.Text = ""
                    tableflag.patera = false
                else
                    PateraCheckBox.Text = "v"
					tableflag.patera = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
RupesLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Rupes (RU)")) -- Exogeology: cliffs, large rocks
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-12), locsLabelBoxWidth+72, 20+19*12);

RupesCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(RupesLabelBox, 2, 10, 147, 4)

RupesCheckBox.Customdraw =
    function(this)
                if tableflag.rupes then							-- state in the table 
                    RupesCheckBox.Text = "v"
                else
                    RupesCheckBox.Text = ""
                end
    end;
       
RupesCheckBox.Action = (function()
        return
            function()
                if tableflag.rupes then
                    RupesCheckBox.Text = ""
                    tableflag.rupes = false
                else
                    RupesCheckBox.Text = "v"
					tableflag.rupes = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
TesseraLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Area fractured (TE)")) -- Exogeology: ancient land billed and folded
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-13), locsLabelBoxWidth+72, 20+19*13);

TesseraCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(TesseraLabelBox, 2, 10, 147, 4)

TesseraCheckBox.Customdraw =
    function(this)
                if tableflag.tessera then							-- state in the table 
                    TesseraCheckBox.Text = "v"
                else
                    TesseraCheckBox.Text = ""
                end
    end;
       
TesseraCheckBox.Action = (function()
        return
            function()
                if tableflag.tessera then
                    TesseraCheckBox.Text = ""
                    tableflag.tessera = false
                else
                    TesseraCheckBox.Text = "v"
					tableflag.tessera = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
RegioLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Regio (RE)")) -- Exogeology: region (but corresponds to continents on Earth)
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-14), locsLabelBoxWidth+72, 20+19*14);

RegioCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(RegioLabelBox, 2, 10, 147, 4)

RegioCheckBox.Customdraw =
    function(this)
                if tableflag.regio then							-- state in the table 
                    RegioCheckBox.Text = "v"
                else
                    RegioCheckBox.Text = ""
                end
    end;
       
RegioCheckBox.Action = (function()
        return
            function()
                if tableflag.regio then
                    RegioCheckBox.Text = ""
                    tableflag.regio = false
                else
                    RegioCheckBox.Text = "v"
					tableflag.regio = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
ChaosLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Area chaotic (CH)")) -- Exogeology: chaotic terrain
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, renderlocsLeftMargin-2, 10+19*(nlines-15), locsLabelBoxWidth+72, 20+19*15);

ChaosCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(ChaosLabelBox, 2, 10, 147, 4)

ChaosCheckBox.Customdraw =
    function(this)
                if tableflag.chaos then							-- state in the table 
                    ChaosCheckBox.Text = "v"
                else
                    ChaosCheckBox.Text = ""
                end
    end;
       
ChaosCheckBox.Action = (function()
        return
            function()
                if tableflag.chaos then
                    ChaosCheckBox.Text = ""
                    tableflag.chaos = false
                else
                    ChaosCheckBox.Text = "v"
					tableflag.chaos = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

--================================= 2e colonne ==========================================

----------------------------------------------------------------------------------------
TerraLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
   -- :bordercolor(cbubordoff)      
   -- :fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Land, Country (TA)")) -- Exogeology: vast fields, land; on Earth used for country
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-1), locsLabelBoxWidth-98, 20+19*1);

TerraCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(TerraLabelBox, 2, 10, 147, 4)

TerraCheckBox.Customdraw =
    function(this)
                if tableflag.terra then							-- state in the table 
                    TerraCheckBox.Text = "v"
                else
                    TerraCheckBox.Text = ""
                end
    end;
       
TerraCheckBox.Action = (function()
        return
            function()
                if tableflag.terra then
                    TerraCheckBox.Text = ""
                    tableflag.terra = false
                else
                    TerraCheckBox.Text = "v"
					tableflag.terra = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
CoronaLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Magmatic plume(CR)")) -- Exogeology: rising of mantellic plumes (magma)
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-2), locsLabelBoxWidth-98, 20+19*2);

CoronaCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(CoronaLabelBox, 2, 10, 147, 4)

CoronaCheckBox.Customdraw =
    function(this)
                if tableflag.corona then							-- state in the table 
                    CoronaCheckBox.Text = "v"
                else
                    CoronaCheckBox.Text = ""
                end
    end;
       
CoronaCheckBox.Action = (function()
        return
            function()
                if tableflag.corona then
                    CoronaCheckBox.Text = ""
                    tableflag.corona = false
                else
                    CoronaCheckBox.Text = "v"
					tableflag.corona = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
DorsumLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Dorsal (DO)")) -- Exogeology: back ridges
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-3), locsLabelBoxWidth-98, 20+19*3);

DorsumCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(DorsumLabelBox, 2, 10, 147, 4)

DorsumCheckBox.Customdraw =
    function(this)
                if tableflag.dorsum then							-- state in the table 
                    DorsumCheckBox.Text = "v"
                else
                    DorsumCheckBox.Text = ""
                end
    end;
       
DorsumCheckBox.Action = (function()
        return
            function()
                if tableflag.dorsum then
                    DorsumCheckBox.Text = ""
                    tableflag.dorsum = false
                else
                    DorsumCheckBox.Text = "v"
					tableflag.dorsum = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
RimaLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Rift (RI)")) -- Exogeology: crevasses, rifts
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-4), locsLabelBoxWidth-98, 20+19*4);

RimaCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(RimaLabelBox, 2, 10, 147, 4)

RimaCheckBox.Customdraw =
    function(this)
                if tableflag.rima then							-- state in the table 
                    RimaCheckBox.Text = "v"
                else
                    RimaCheckBox.Text = ""
                end
    end;
       
RimaCheckBox.Action = (function()
        return
            function()
                if tableflag.rima then
                    RimaCheckBox.Text = ""
                    tableflag.rima = false
                else
                    RimaCheckBox.Text = "v"
					tableflag.rima = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
OtherLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Other (XX)")) -- others features in Celestia render
    :movable(false)
    :active(true) 
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-5), locsLabelBoxWidth-98, 20+19*5);

OtherCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(OtherLabelBox, 2, 10, 147, 4)

OtherCheckBox.Customdraw =
    function(this)
                if tableflag.other then							-- state in the table 
                    OtherCheckBox.Text = "v"
                else
                    OtherCheckBox.Text = ""
                end
    end;
       
OtherCheckBox.Action = (function()
        return
            function()
                if tableflag.other then
                    OtherCheckBox.Text = ""
                    tableflag.other = false
                else
                    OtherCheckBox.Text = "v"
					tableflag.other = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();


-----------------------------------------------------------------------------------------
AstrumLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Astrum (AS)")) -- Radial-patterned features on Venus
    :movable(false)
    :active(true) 
	-- 3eme colonne at left +300, at right -300
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-6), locsLabelBoxWidth-98, 20+19*6);

AstrumCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(AstrumLabelBox, 2, 10, 147, 4)

AstrumCheckBox.Customdraw =
    function(this)
                if tableflag.astrum then							-- state in the table 
                    AstrumCheckBox.Text = "v"
                else
                    AstrumCheckBox.Text = ""
                end
    end;
       
AstrumCheckBox.Action = (function()
        return
            function()
                if tableflag.astrum then
                    AstrumCheckBox.Text = ""
                    tableflag.astrum = false
                else
                    AstrumCheckBox.Text = "v"
					tableflag.astrum = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
CatenaLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Catena (CA)")) -- A chain of craters
    :movable(false)
    :active(true) 
	-- 3eme colonne at left +300, at right -300
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-7), locsLabelBoxWidth-98, 20+19*7);

CatenaCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(CatenaLabelBox, 2, 10, 147, 4)

CatenaCheckBox.Customdraw =
    function(this)
                if tableflag.catena then							-- state in the table 
                    CatenaCheckBox.Text = "v"
                else
                    CatenaCheckBox.Text = ""
                end
    end;
       
CatenaCheckBox.Action = (function()
        return
            function()
                if tableflag.catena then
                    CatenaCheckBox.Text = ""
                    tableflag.catena = false
                else
                    CatenaCheckBox.Text = "v"
					tableflag.catena = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
FossaLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Fossa (FO)")) -- Long, narrow, shallow depression
    :movable(false)
    :active(true) 
	-- 3eme colonne at left +300, at right -300
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-8), locsLabelBoxWidth-98, 20+19*8);

FossaCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(FossaLabelBox, 2, 10, 147, 4)

FossaCheckBox.Customdraw =
    function(this)
                if tableflag.fossa then							-- state in the table 
                    FossaCheckBox.Text = "v"
                else
                    FossaCheckBox.Text = ""
                end
    end;
       
FossaCheckBox.Action = (function()
        return
            function()
                if tableflag.fossa then
                    FossaCheckBox.Text = ""
                    tableflag.fossa = false
                else
                    FossaCheckBox.Text = "v"
					tableflag.fossa = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
FluctusLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Fluctus (FL)")) -- Terrain covered by outflow of liquid. Used on Venus, Io and Titan.
    :movable(false)
    :active(true) 
	-- 3eme colonne at left +300, at right -300
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-9), locsLabelBoxWidth-98, 20+19*9);

FluctusCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(FluctusLabelBox, 2, 10, 147, 4)

FluctusCheckBox.Customdraw =
    function(this)
                if tableflag.fluctus then							-- state in the table 
                    FluctusCheckBox.Text = "v"
                else
                    FluctusCheckBox.Text = ""
                end
    end;
       
FluctusCheckBox.Action = (function()
        return
            function()
                if tableflag.fluctus then
                    FluctusCheckBox.Text = ""
                    tableflag.fluctus = false
                else
                    FluctusCheckBox.Text = "v"
					tableflag.fluctus = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
FarrumLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Farrum (FR)")) -- Pancake-like structure, or a row of such structures. Used only on Venus.
    :movable(false)
    :active(true) 
	-- 3eme colonne at left +300, at right -300
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-10), locsLabelBoxWidth-98, 20+19*10);

FarrumCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(FarrumLabelBox, 2, 10, 147, 4)

FarrumCheckBox.Customdraw =
    function(this)
                if tableflag.farrum then							-- state in the table 
                    FarrumCheckBox.Text = "v"
                else
                    FarrumCheckBox.Text = ""
                end
    end;
       
FarrumCheckBox.Action = (function()
        return
            function()
                if tableflag.farrum then
                    FarrumCheckBox.Text = ""
                    tableflag.farrum = false
                else
                    FarrumCheckBox.Text = "v"
					tableflag.farrum = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
LineaLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Linea (LI)")) -- Small dark spots on Europa
    :movable(false)
    :active(true) 
	-- 3eme colonne at left +300, at right -300
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-11), locsLabelBoxWidth-98, 20+19*11);

LineaCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(LineaLabelBox, 2, 10, 147, 4)

LineaCheckBox.Customdraw =
    function(this)
                if tableflag.linea then							-- state in the table 
                    LineaCheckBox.Text = "v"
                else
                    LineaCheckBox.Text = ""
                end
    end;
       
LineaCheckBox.Action = (function()
        return
            function()
                if tableflag.linea then
                    LineaCheckBox.Text = ""
                    tableflag.linea = false
                else
                    LineaCheckBox.Text = "v"
					tableflag.linea = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
MensaLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Mensa (MN)")) -- A flat-topped prominence with cliff-like edges, i.e. a mesa.
    :movable(false)
    :active(true) 
	-- 3eme colonne at left +300, at right -300
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-12), locsLabelBoxWidth-98, 20+19*12);

MensaCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(MensaLabelBox, 2, 10, 147, 4)

MensaCheckBox.Customdraw =
    function(this)
                if tableflag.mensa then							-- state in the table 
                    MensaCheckBox.Text = "v"
                else
                    MensaCheckBox.Text = ""
                end
    end;
       
MensaCheckBox.Action = (function()
        return
            function()
                if tableflag.mensa then
                    MensaCheckBox.Text = ""
                    tableflag.mensa = false
                else
                    MensaCheckBox.Text = "v"
					tableflag.mensa = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
PlanitiaLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Planitia (PL)")) -- Low plain e.g. Amazonis Planitia.
    :movable(false)
    :active(true) 
	-- 3eme colonne at left +300, at right -300
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-13), locsLabelBoxWidth-98, 20+19*13);

PlanitiaCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(PlanitiaLabelBox, 2, 10, 147, 4)

PlanitiaCheckBox.Customdraw =
    function(this)
                if tableflag.planitia then							-- state in the table 
                    PlanitiaCheckBox.Text = "v"
                else
                    PlanitiaCheckBox.Text = ""
                end
    end;
       
PlanitiaCheckBox.Action = (function()
        return
            function()
                if tableflag.planitia then
                    PlanitiaCheckBox.Text = ""
                    tableflag.planitia = false
                else
                    PlanitiaCheckBox.Text = "v"
					tableflag.planitia = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
RectilumLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Rectilum (RT)")) -- reticular (netlike) pattern on Venus
    :movable(false)
    :active(true) 
	-- 3eme colonne at left +300, at right -300
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-14), locsLabelBoxWidth-98, 20+19*14);

RectilumCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(RectilumLabelBox, 2, 10, 147, 4)

RectilumCheckBox.Customdraw =
    function(this)
                if tableflag.rectilum then							-- state in the table 
                    RectilumCheckBox.Text = "v"
                else
                    RectilumCheckBox.Text = ""
                end
    end;
       
RectilumCheckBox.Action = (function()
        return
            function()
                if tableflag.rectilum then
                    RectilumCheckBox.Text = ""
                    tableflag.rectilum = false
                else
                    RectilumCheckBox.Text = "v"
					tableflag.rectilum = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
UndaeLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(normalfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(17, 4)
    :text(_("Undae (UN)")) -- A field of dunes. Used on Venus, Mars and Titan.
    :movable(false)
    :active(true) 
	-- 3eme colonne at left +300, at right -300
    :attach(renderLocsFrame, 170+renderlocsLeftMargin-2, 10+19*(nlines-15), locsLabelBoxWidth-98, 20+19*15);

UndaeCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(featckeckcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(UndaeLabelBox, 2, 10, 147, 4)

UndaeCheckBox.Customdraw =
    function(this)
                if tableflag.undae then							-- state in the table 
                    UndaeCheckBox.Text = "v"
                else
                    UndaeCheckBox.Text = ""
                end
    end;
       
UndaeCheckBox.Action = (function()
        return
            function()
                if tableflag.undae then
                    UndaeCheckBox.Text = ""
                    tableflag.undae = false
                else
                    UndaeCheckBox.Text = "v"
					tableflag.undae = true
                end
                obs:setlocationflags(tableflag)
            end
        end) ();

----------------------------------------------------------------------------------------
]]

--[[-- liste of features
	    -- col. 1
			"city",			--01-- villes
			"observatory",		--02-- observatoires
			"boundary",			--03-- fronti鳥s
			"landingsite",		--04-- LF sites d'atterrissage Lune
			"landing site",		--00--    sites d'atterrissage Lune
			"crater",			--05-- AA crat鳥s
			"vallis",			--06-- VA vall꦳
			"mons",				--07-- MO montagnes
			"planum",			--08-- PM plateaux
			"mare",				--09-- ME mers et ocꢮs
            -- col. 2
			"chasma",			--10-- CM exogeologie : canyon	
			"patera",			--11-- PE exogeologie : volcans extra-terrestre, caldera, crat鳥s
			"rupes",			--12-- RU exogeologie : falaises, gros rochers
			"tessera",			--13-- TE exogeologie : terrains anciens factur고et pli곍
			"regio",			--14-- RE exogeologie : rꨩon
			"chaos",			--15-- CH exogeologie : terrain chaotique
			"terra",			--16-- TA exogeologie : vastes terrains, country
			"corona",			--17-- CR exogeologie : remont꦳ de panaches mantelliques (magma)
			"dorsum",			--18-- DO exogeologie : cr뵥s dorsales
			"rima",				--19-- RI exogeologie : crevasses, rifts
			"other",			--20-- XX autres
			-- col. 3
			"astrum",			--21-- AS Radial-patterned features on Venus
			"catena",			--22-- CA A chain of craters
			"fossa",			--23-- FO Long, narrow, shallow depression
			"fluctus",			--24-- FL Terrain covered by outflow of liquid. Used on Venus, Io and Titan.
			"farrum",			--25-- FR Pancake-like structure, or a row of such structures. Used only on Venus.
			"linea",			--26-- LI Small dark spots on Europa
			"mensa",			--27-- MN A flat-topped prominence with cliff-like edges, i.e. a mesa.
			"planitia",			--28-- PL Low plain e.g. Amazonis Planitia.
			"rectilum",			--29-- RT reticular (netlike) pattern on Venus
			"undae",			--30-- UN A field of dunes. Used on Venus, Mars and Titan.
	 -- other name of features
			"Vastitas"			--31-- VS An extensive plain. The only feature with this designation is Vastitas Borealis.
			"Virga"				--32-- VI A streak or stripe of color. This term is currently used only on Titan.
			"Tholus"			--33-- TH Small domical mountain or hill e.g. Hecates Tholus.
			"Sulcus"			--34-- SU Subparallel furrows and ridges
			"Sinus"				--35-- SI "Bay"; small plain on Moon or Mars, e.g. Sinus Meridiani; On Titan, bay within bodies of liquid.
			"Serpens"			--36-- SE Sinuous feature with segments of positive and negative relief along its length
			"Scopulus"			--37-- SC Lobate or irregular scarp
			"Satellite feature" --38-- SF A feature that shares the name of an associated feature, for example Hertzsprung D.
			"Promontorium"		--39-- PR "Cape"; headland. Used only on the Moon.
			"Plume"				--40-  PU A cryovolcanic feature on Triton. This term is currently unused.
			"Palus"				--41-- PA "Swamp"; small plain. Used on the Moon and Mars.
			"Oceanus"			--42-- OC Very large dark area. The only feature with this designation is Oceanus Procellarum.
			"Macula"			--43-- MA Dark spot, may be irregular
			"Lenticula"			--44-- LE Small dark spots on Europa
			"Large ringed feature" --45-- LG Cryptic ringed features
			"Landing site name"	--46-- LF Lunar features at or near Apollo landing sites
			"Lacus"				--47-- LC A "lake" or small plain on Moon and Mars; on Titan, a "lake" or small, dark plain with discrete, sharp boundaries.
			"Lacuna"			--48-- LU Irregularly shaped depression having the appearance of a dry lake bed. Used only on Titan.
			"Labyrinthus"		--49-- LB Complex of intersecting valleys or ridges.
			"Labes"				--50-- LA Landslide debris. Used only on Mars.
			"Insula"			--51-- IN Island (islands), an isolated land area (or group of such areas) surrounded by, or nearly surrounded by, a liquid area (sea or lake). Used only on Titan.
			"Fretum"			--52-- FT Strait of liquid connecting two larger areas of liquid. Used only on Titan.
			"Flumen"			--53-- FM Channel on Titan that might carry liquid
			"Flexus"			--54-- FE Very low curvilinear ridge with a scalloped pattern
			"Facula"			--55-- FA Bright spot
			"Eruptive center"	--56-- ER An active volcano on Io. This term is implicit.
			"Colles"			--57-- CO A collection of small hills or knobs.
			"Cavus"				--58-- CB Hollows, irregular steep-sided depressions usually in arrays or clusters
			"Arcus"				--59-- AR Arc: curved feature
			"Albedo feature"	--60-- AL An area which shows a contrast in brightness or darkness (albedo) with adjacent areas. This term is implicit.
]]	